@@ -1,5 +1,11 @@
 """TurboGears Front-to-Back Web Framework"""
 
+__all__ = ['absolute_url', 'database', 'command', 'config',
+    'controllers', 'expose', 'flash', 'error_handler',
+    'exception_handler', 'mochikit', 'redirect',
+    'scheduler', 'start_server', 'update_config',
+    'url', 'validate', 'validators', 'view', 'widgets']
+
 import warnings
 
 import pkg_resources
@@ -12,15 +18,16 @@ from turbogears.controllers import (absolute_url, expose, flash, validate,
 from turbogears.paginate import paginate
 from turbogears.widgets import mochikit, jsi18nwidget
 from turbogears.startup import start_server
-from turbogears import (config, controllers, view, database, validators, command,
-    i18n, widgets, startup, scheduler)
+from turbogears import (config, controllers, view, database, validators,
+    command, i18n, widgets, startup, scheduler)
+
 
 # load global symbols for TG extensions (currently only used by tgfastdata)
-extensions = pkg_resources.iter_entry_points("turbogears.extensions")
+extensions = pkg_resources.iter_entry_points('turbogears.extensions')
 for entrypoint in extensions:
     try:
         extension = entrypoint.load()
-        if hasattr(extension, "tgsymbols"):
+        if hasattr(extension, 'tgsymbols'):
             globals().update(extension.tgsymbols())
     except Exception, exception:
         warnings.warn("Could not load extension %s from %s: %s"
@@ -28,25 +35,3 @@ for entrypoint in extensions:
 
 i18n.install() # adds _ (gettext) to builtins namespace
 
-
-__all__ = [
-    "absolute_url",
-    "database",
-    "command",
-    "config",
-    "controllers",
-    "expose",
-    "flash",
-    "error_handler",
-    "exception_handler",
-    "mochikit",
-    "redirect",
-    "scheduler",
-    "start_server",
-    "update_config",
-    "url",
-    "validate",
-    "validators",
-    "view",
-    "widgets",
-]
