@@ -416,7 +416,8 @@ def _execute_func(func, template, format, content_type, fragment, options,
         # add special parameters again
         kw.update(tg_kw)
 
-    if config.get('environment') != 'production':
+    env = config.get('environment') or 'development'
+    if env == 'development':
         # Only output this in development mode: If it's a field storage object,
         # this means big memory usage, and we don't want that in production
         log.debug("Calling %s with *(%s), **(%s)", func, args, kw)
