@@ -71,7 +71,7 @@ if sqlalchemy:
         alch_args = dict()
         for k, v in config.items():
             if 'sqlalchemy' in k:
-                alch_args[k.split('.')[-1]] = v
+                alch_args[k.split('.', 1)[-1]] = v
 
         try:
             dburi = alch_args.pop('dburi')
@@ -89,8 +89,8 @@ if sqlalchemy:
 
         for k, v in config.items():
             if '.dburi' in k and 'sqlalchemy.' not in k:
-                get_metadata(k.split('.')[0]).bind = sqlalchemy.create_engine(
-                    v, **alch_args)
+                get_metadata(k.split('.', 1)[0]
+                    ).bind = sqlalchemy.create_engine(v, **alch_args)
 
     def create_session():
         """Create a session that uses the engine from thread-local metadata.
