@@ -14,7 +14,7 @@ tg_domain_table = Table('tg_i18n_domain', metadata,
 tg_message_table = Table('tg_i18n_message', metadata,
     Column('id', Integer, primary_key=True),
     Column('name', Unicode),
-    Column('text', Unicode, default=u""),
+    Column('text', Unicode, default=u''),
     Column('domain_id', Integer, ForeignKey(tg_domain_table.c.id)),
     Column('locale', String(length=15)),
     Column('created', DateTime, default=datetime.now),
