@@ -1,27 +1,19 @@
 """The TurboGears identity management package."""
 
 # declare what should be exported
-__all__ = [
-    '_encrypt_password',
-    'create_default_provider',
-    'current',
-    'current_provider',
-    'encrypt_password',
-    'encrypt_pw_with_algorithm',
-    'get_identity_errors',
-    'get_failure_url',
-    'set_current_identity',
-    'set_current_provider',
-    'set_identity_errors',
-    'set_login_attempted',
-    'was_login_attempted',
-]
+__all__ = ['_encrypt_password', 'create_default_provider',
+    'current', 'current_provider',
+    'encrypt_password', 'encrypt_pw_with_algorithm',
+    'get_identity_errors', 'get_failure_url',
+    'set_current_identity', 'set_current_provider',
+    'set_identity_errors', 'set_login_attempted',
+    'was_login_attempted', 'verify_identity_status']
 
 
 import logging
 try:
     from hashlib import md5, sha1
-except ImportError:
+except ImportError: # Python < 2.5
     from sha import new as sha1
     from md5 import new as md5
 
@@ -30,7 +22,8 @@ import pkg_resources
 import turbogears
 
 from turbogears.util import deprecated, request_available, load_class
-from turbogears.identity.exceptions import *
+from turbogears.identity.exceptions import (IdentityConfigurationException,
+    RequestRequiredException, IdentityManagementNotEnabledException)
 
 
 log = logging.getLogger('turbogears.identity')
@@ -46,25 +39,29 @@ def create_default_provider():
     raises an IdentityConfigurationException.
 
     """
-    provider_plugin = turbogears.config.get('identity.provider', 'sqlobject')
+    provider_plugin = turbogears.config.get('identity.provider', 'sqlalchemy')
     plugins = pkg_resources.iter_entry_points(
         'turbogears.identity.provider', provider_plugin)
 
     log.debug("Loading provider from plugin: %s", provider_plugin)
 
-    provider_class = None
     for entrypoint in plugins:
-        provider_class = entrypoint.load()
+        try:
+            provider_class = entrypoint.load()
+        except Exception, load_error:
+            raise IdentityConfigurationException(
+                "IdentityProvider plugin can't be loaded: %s\n%s"
+                % (provider_plugin, load_error))
         break
-
-    if not provider_class:
+    else:
         provider_class = load_class(provider_plugin)
 
     if not provider_class:
         raise IdentityConfigurationException(
-            "IdentityProvider plugin missing: %s" % provider_plugin )
-    else:
-        return provider_class()
+            "IdentityProvider plugin missing: %s" % provider_plugin)
+
+    return provider_class()
+
 
 def was_login_attempted():
     try:
@@ -72,17 +69,19 @@ def was_login_attempted():
     except AttributeError:
         return False
 
+
 def set_login_attempted(flag):
     cherrypy.request.identity_login_attempted = flag
 
+
 def set_current_identity(identity):
     cherrypy.request.identity = identity
-
     try:
-        cherrypy.request.user_name = identity.user_name
-
+        user_name = identity.user_name
     except AttributeError:
-        cherrypy.request.user_name = None
+        user_name = None
+    cherrypy.request.login = cherrypy.request.user_name = user_name
+
 
 def set_current_provider(provider):
     cherrypy.request.identityProvider = provider
@@ -118,7 +117,7 @@ def encrypt_pw_with_algorithm(algorithm, password):
         if custom_encryption:
             hashed_password = custom_encryption(password_8bit)
     # Make sure the hashed password is a unicode object at the end of the
-    # process, because SQLAlchemy _wants_ that for Unicode columns.
+    # process, because SQLAlchemy _wants_ that for unicode columns.
     if not isinstance(hashed_password, unicode):
         hashed_password = hashed_password.decode('utf-8')
     return hashed_password
@@ -127,6 +126,7 @@ _encrypt_password = deprecated(
     "Use identity.encrypt_pw_with_algorithm instead."
 )(encrypt_pw_with_algorithm)
 
+
 def encrypt_password(cleartext):
     return current_provider.encrypt_password(cleartext)
 
@@ -142,7 +142,6 @@ class IdentityWrapper(object):
     def identity(self):
         try:
             identity = cherrypy.request.identity
-
         except AttributeError:
             identity = None
 
@@ -177,7 +176,6 @@ class ProviderWrapper(object):
     def __getattr__(self, name):
         try:
             provider = cherrypy.request.identityProvider
-
         except AttributeError:
             try:
                 provider = create_default_provider()
@@ -193,5 +191,31 @@ class ProviderWrapper(object):
 
         return getattr(provider, name)
 
+
 current = IdentityWrapper()
 current_provider = ProviderWrapper()
+
+
+def verify_identity_status():
+    """A tool that sets response status based on identity's success or failure.
+
+    This is necessary since the status will be overridden by the result of
+    forwarding the user to the login page.
+
+    Does not override status if the login controller errors out.
+
+    """
+    status = cherrypy.response.status
+    if not status or str(status) < '400':
+        new_status = cherrypy.request.wsgi_environ.get('identity.status')
+        if new_status:
+            cherrypy.response.status = new_status
+        auth_realm = cherrypy.request.wsgi_environ.get('identity.auth_realm')
+        if auth_realm:
+            cherrypy.response.headers['WWW-Authenticate'] = auth_realm
+            # During the finalize stage, CP stops looking at response.headers,
+            #  so populate header_list as well.
+            if not cherrypy.response.header_list:
+                cherrypy.response.header_list = []
+            cherrypy.response.header_list.append(('WWW-Authenticate', auth_realm))
+
