@@ -1,27 +1,13 @@
 """Definition of the identity predicates."""
 
-
 # declare what should be exported
-__all__ = [
-    'All',
-    'Any',
-    'NotAny',
-    'CompoundPredicate',
-    'IdentityPredicateHelper',
-    'Predicate',
-    'SecureObject',
-    'SecureResource',
-    'from_host',
-    'from_any_host',
-    'in_all_groups',
-    'in_any_group',
-    'in_group',
-    'has_all_permissions',
-    'has_any_permission',
-    'has_permission',
-    'not_anonymous',
-    'require',
-]
+__all__ = ['All', 'Any', 'NotAny', 'CompoundPredicate',
+    'IdentityPredicateHelper', 'Predicate',
+    'SecureObject', 'SecureResource',
+    'from_host', 'from_any_host',
+    'in_all_groups', 'in_any_group', 'in_group',
+    'has_all_permissions', 'has_any_permission', 'has_permission',
+    'not_anonymous', 'require']
 
 
 from types import MethodType
@@ -30,9 +16,10 @@ from cherrypy import request
 
 from turbogears import config
 from turbogears.decorator import weak_signature_decorator
-from turbogears.util import match_ip
+from turbogears.util import match_ip, request_available
 
-from turbogears.identity.exceptions import *
+from turbogears.identity.exceptions import (IdentityException,
+    IdentityFailure, RequestRequiredException)
 from turbogears.identity.base import current
 
 
@@ -195,12 +182,11 @@ class has_any_permission(Any, IdentityPredicateHelper):
 
 
 def _remoteHost():
-    try:
+    if not request_available():
+        raise RequestRequiredException()
+    else:
         return request.headers.get('X-Forwarded-For', request.headers.get(
             'Remote-Addr', '')).rsplit(',', 1)[-1].strip() or None
-    except AttributeError:
-        raise RequestRequiredException()
-
 
 class from_host(Predicate, IdentityPredicateHelper):
     """Predicate for checking whether the visitor's host is a permitted host.
