@@ -1,36 +1,39 @@
 """The visit and identity management *plugins* are defined here."""
 
-__all__ = [
-    'IdentityVisitPlugin',
-    'create_extension_model',
-    'shutdown_extension',
-    'start_extension',
-]
+__all__ = ['IdentityVisitPlugin', 'create_extension_model',
+    'shutdown_extension', 'start_extension']
 
 import base64
 import logging
 
 from cherrypy import request
+from formencode.variabledecode import variable_decode, variable_encode
 
 from turbogears import config, visit
 from turbogears.identity import (create_default_provider, set_current_identity,
     set_current_provider, set_login_attempted)
-from turbogears.util import pop_request_params
 
-from turbogears.identity.exceptions import *
+from turbogears.identity.exceptions import (IdentityConfigurationException,
+    IdentityException, IdentityFailure)
 
 
-log = logging.getLogger("turbogears.identity")
+log = logging.getLogger('turbogears.identity')
+
+
+# Global Visit plugin
+_plugin = None
 
 
 # Interface for the TurboGears extension
 
 def start_extension():
-    """Register the IdentityVisitPluguin with the visit plug-in framework.
+    """Register the IdentityVisitPlugin with the visit plugin framework.
 
     Also sets up the configured Identity provider.
 
     """
+    global _plugin
+
     # Bail out if the application hasn't enabled this extension
     if not config.get('identity.on', False):
         return
@@ -38,25 +41,38 @@ def start_extension():
     # Identity requires that Visit tracking be enabled
     if not config.get('visit.on', False):
         raise IdentityConfigurationException(
-                "Visit tracking must be enabled (visit.on)")
+            "Visit tracking must be enabled (via visit.on).")
+
+    # Bail out if Visit tracking plugin is already registered
+    if _plugin:
+        log.info("Identity already started")
+        return
 
     log.info("Identity starting")
     # Temporary until tg-admin can call create_extension_model
     create_extension_model()
-    # Register the plugin for the Visit Tracking framework
-    visit.enable_visit_plugin(IdentityVisitPlugin())
+    # Create and register the plugin for the Visit Tracking framework
+    _plugin = IdentityVisitPlugin()
+    visit.enable_visit_plugin(_plugin)
 
 
 def shutdown_extension():
-    """Stops the IdentityVisitPlugin.
+    """Stops the IdentityVisitPlugin."""
+    global _plugin
 
-    This is practically a no-op currently.
-
-    """
     # Bail out if the application hasn't enabled this extension
     if not config.get('identity.on', False):
         return
-    log.info("Identity shutting down")
+
+    # Bail out if the Visit tracking plugin is already unregistered
+    if not _plugin:
+        log.info("Identity already shut down")
+        return
+
+    # Unregister the plugin for the Visit Tracking framework
+    visit.disable_visit_plugin(_plugin)
+    _plugin = None
+    log.info("Identity has been shut down.")
 
 
 def create_extension_model():
@@ -66,7 +82,7 @@ def create_extension_model():
 
 
 class IdentityVisitPlugin(object):
-    """Visit plug-in tying the Identity framework to the visit management."""
+    """Visit plugin tying the Identity framework to the visit management."""
 
     def __init__(self):
         log.info("Identity visit plugin initialized")
@@ -77,9 +93,9 @@ class IdentityVisitPlugin(object):
         # When retrieving identity information from the form, use the following
         # form field names. These fields will be removed from the post data to
         # prevent the controller from receiving unexpected fields.
-        self.user_name_field = get('identity.form.user_name', 'user_name').split('.')
-        self.password_field = get('identity.form.password', 'password').split('.')
-        self.submit_button_name = get('identity.form.submit', 'login').split('.')
+        self.user_name_field = get('identity.form.user_name', 'user_name')
+        self.password_field = get('identity.form.password', 'password')
+        self.submit_button_name = get('identity.form.submit', 'login')
 
         # Sources for identity information and the order in which they should be
         # checked. These terms are mapped to methods by prepending
@@ -102,8 +118,8 @@ class IdentityVisitPlugin(object):
                 try:
                     source_method = getattr(self, 'identity_from_' + s)
                 except AttributeError:
-                    raise IdentityConfigurationException(
-                        "Invalid identity source: %s" % s)
+                    raise IdentityConfigurationException("Invalid "
+                        "identity source: %s (check identity.source)" % s)
                 self.identity_sources.append(source_method)
 
     def identity_from_request(self, visit_key):
@@ -181,13 +197,14 @@ class IdentityVisitPlugin(object):
 
         """
         # only try to process credentials for login forms
-        if pop_request_params(self.submit_button_name) is None:
+        params = request.params
+        if params.pop(self.submit_button_name, None) is None:
             return None
         # form data contains login credentials
-        pop_request_params(self.submit_button_name + ['x'])
-        pop_request_params(self.submit_button_name + ['y'])
-        user_name = pop_request_params(self.user_name_field)
-        password = pop_request_params(self.password_field)
+        params.pop(self.submit_button_name + '.x', None)
+        params.pop(self.submit_button_name + '.y', None)
+        user_name = params.pop(self.user_name_field, None)
+        password = params.pop(self.password_field, None)
         if user_name is None:
             log.error("Missing user name in login form")
             return None
@@ -210,13 +227,16 @@ class IdentityVisitPlugin(object):
 
     def record_request(self, visit):
         """Authenticate request and try to associate the visit with an identity."""
-        # This method is called by the visit plug-in mechanism on each request with a visit key.
+        # This method is called by the visit plugin mechanism on each request with a visit key.
         # default to keeping the identity filter off
-        if not config.get('identity.on', True):
+        if not config.get('identity.on', False):
             log.debug("Identity is not enabled. Setting current identity to None")
             set_current_identity(None)
             return
-
+        if 'identity.path_info' in request.wsgi_environ:
+            # restore path_info and params after internal redirect
+            request.path_info = request.wsgi_environ.pop('identity.path_info')
+            request.params = request.wsgi_environ.pop('identity.params', {})
         try:
             identity = self.identity_from_request(visit.key)
         except IdentityException, e:
@@ -228,3 +248,4 @@ class IdentityVisitPlugin(object):
         # stash the user in the thread data for this request
         set_current_identity(identity)
         set_current_provider(self.provider)
+
