@@ -9,6 +9,7 @@ import atexit
 import logging
 import os
 import signal
+import sys
 
 from os.path import abspath, exists
 
@@ -30,10 +31,11 @@ from turbogears.hooks import NestedVariablesHook
 
 log = logging.getLogger("turbogears.startup")
 
-DNS_SD_PID = None
+dns_sd_pid = None
 call_on_startup = []
 call_on_shutdown = []
 webpath = ''
+started = False
 
 
 # module public functions
@@ -45,9 +47,13 @@ def start_bonjour(package=None):
     daemon (Linux etc.) is available or the 'dns-sd' program (Mac OS X).
 
     """
-    global DNS_SD_PID
-    if DNS_SD_PID:
+    global dns_sd_pid
+    if dns_sd_pid:
         return
+    if sys.platform in ('win32', 'os2'):
+        dns_sd_pid = -1 # service not available
+        return
+
     if not package:
         app = cherrypy.tree.apps.get('')
         if not app:
@@ -57,30 +63,32 @@ def start_bonjour(package=None):
 
     host = config.get('server.socket_host', '0.0.0.0')
     port = str(config.get('server.socket_port'))
-    env = config.get('environment', 'development')
-    name = package + ': ' + env
-    type = '_http._tcp'
-
-    cmds = [['/usr/bin/avahi-publish-service', ['-H', host, name, type, port]],
-            ['/usr/bin/dns-sd', ['-R', name, type, '.' + host, port, 'path=/']]]
+    env = config.get('environment') or 'development'
+    name = '%s:%s' % (package, env)
+    typ = '_http._tcp'
+    cmds = [
+        ('/usr/bin/avahi-publish-service', ['-H', host, name, typ, port]),
+        ('/usr/bin/dns-sd', ['-R', name, typ, '.' + host, port, 'path=/'])]
 
     for cmd, args in cmds:
-        # TODO:. This check is flawed.  If one has both services installed and
+        # TODO: This check is flawed.  If one has both services installed and
         # avahi isn't the one running, then this won't work.  We should either
         # try registering with both or checking what service is running and use
-        # that.  Program availability on the filesystem was never enough...
+        # that.  Program availability on the file system was never enough...
         if exists(cmd):
-            DNS_SD_PID = os.spawnv(os.P_NOWAIT, cmd, [cmd]+args)
+            dns_sd_pid = os.spawnv(os.P_NOWAIT, cmd, [cmd] + args)
             atexit.register(stop_bonjour)
             break
+    else:
+        dns_sd_pid = -1 # service not available
 
 
 def stop_bonjour():
     """Stop the Bonjour publishing daemon if it is running."""
-    if not DNS_SD_PID:
+    if not dns_sd_pid or dns_sd_pid < 0:
         return
     try:
-        os.kill(DNS_SD_PID, signal.SIGTERM)
+        os.kill(dns_sd_pid, signal.SIGTERM)
     except OSError:
         pass
 
@@ -120,6 +128,12 @@ def config_root():
         if webpath:
             config.update({'/': {'request.dispatch': VirtualPathDispatcher(
                 config.get('request.dispatch'), webpath)}})
+    if config.get('tg.fancy_exception', False):
+        from paste import evalexception
+        config.update({'request.throw_errors': True, '/': {
+            'wsgi.pipeline': [('evalexc', evalexception.EvalException)],
+            'wsgi.evalexc.global_conf': {},
+            'wsgi.evalexc.xmlhttp_key': "_xml"}})
 
 
 def start_turbogears():
@@ -145,6 +159,10 @@ def start_turbogears():
     * Starts the TurboGears scheduler if enabled in the configuration.
 
     """
+    global started
+    if started:
+        log.info("TurboGears has already been started.")
+        return
     log.info("Starting TurboGears...")
 
     # Initialize template engines and load base templates
@@ -171,10 +189,11 @@ def start_turbogears():
             'before_handler', VisitTool(), priority=62)
 
     # Register server with Bonjour framework
-    bonjoursetting = config.get('tg.bonjour', None)
-    if bonjoursetting or config.get('environment') != 'production':
+    bonjour = config.get('tg.bonjour', None)
+    env = config.get('environment') or 'development'
+    if bonjour or env == 'development':
         log.info("Starting the Bonjour service...")
-        start_bonjour(bonjoursetting)
+        start_bonjour(bonjour)
 
     # Bind metadata for SQLAlchemy
     if config.get('sqlalchemy.dburi'):
@@ -217,6 +236,7 @@ def start_turbogears():
         log.info("Starting the scheduler...")
         scheduler.start_scheduler()
 
+    started = True
     log.info("TurboGears has been started.")
 
 
@@ -227,7 +247,10 @@ def stop_turbogears():
     calls user provided shutdown functions and stops the scheduler.
 
     """
-
+    global started
+    if not started:
+        log.info("TurboGears has already been stopped.")
+        return
     log.info("Stopping TurboGears...")
 
     if config.get('tg.scheduler', False):
@@ -276,22 +299,26 @@ def stop_turbogears():
     except AttributeError:
         log.debug("CherryPy hooks could not be restored.")
 
+    started = False
     log.info("TurboGears has been stopped.")
 
 
 def start_server(root):
     """Start the CherryPy Server."""
+    if started:
+        log.info("The server has already been started.")
+        return
     app = cherrypy.tree.mount(root, config=config.app)
-
-    if config.get('tg.fancy_exception', False):
-        from paste import evalexception
-        app.wsgiapp.pipeline.append(
-            ('paste_exc', evalexception.middleware.EvalException))
-
     config.update({'log.screen': False})
-
-    cherrypy.engine.start()
-    cherrypy.engine.block()
+    embedded = config.get('environment') == 'embedded'
+    if config.get('engine.start', not embedded):
+        cherrypy.engine.start()
+        if config.get('engine.block', not embedded):
+            cherrypy.engine.block()
+    else:
+        start_turbogears()
+        atexit.register(stop_turbogears)
+    return app
 
 
 # Subscribe to engine events at import time so that our callbacks get used
