@@ -314,7 +314,8 @@ class TestRoot(testutil.TGTest):
     def test_js_files(self):
         """Can access the JavaScript files"""
         response = self.app.get('/tg_js/MochiKit.js', status=200)
-        assert response.headers['Content-Type'] in ('text/javascript',
+        assert response.headers['Content-Type'] in (
+            'text/javascript', 'text/x-js',
             'application/javascript', 'application/x-javascript')
 
     def test_json_output(self):
@@ -427,7 +428,7 @@ class TestRoot(testutil.TGTest):
 
     def test_unicode(self):
         response = self.app.get('/unicode')
-        firstline = response.body.split('\n')[0].decode('utf-8')
+        firstline = response.body.split('\n', 1)[0].decode('utf-8')
         assert firstline == u'\u00bfHabla espa\u00f1ol?'
 
     def test_default_format(self):
