@@ -2,12 +2,15 @@
 
 import unittest
 
-import pkg_resources
-from turbogears import view, config
-from turbogears.view.base import _get_plugin_options
-from simplejson import dumps, loads
+try:
+    from json import dumps, loads
+except ImportError: # Python < 2.6
+    from simplejson import dumps, loads
 
+from turbogears import view, config
 from turbogears.view import TGGenshiTemplatePlugin
+from turbogears.view.base import _get_plugin_options
+
 
 html_strict_url = "http://www.w3.org/TR/html4/strict.dtd"
 html_transitional_url = "http://www.w3.org/TR/html4/loose.dtd"
@@ -41,25 +44,22 @@ class TestTemplateEngine(object):
 
 
 class TestView(unittest.TestCase):
-    """Test suite for turbogears.view idependent of external template engines.
+    """Test suite for turbogears.view independent of external template engines.
     """
 
     def setUp(self):
         """Reload the template engines before each run."""
-
         print "Loading all template engines..."
         view.load_engines()
 
     def tearDown(self):
         """Destroy the template engines after each test."""
-
         print "Destroying all template engines..."
         # We need to empty the module global but leave it in place
         view.engines.clear()
 
     def test_view_pass_options(self):
         """Test that view.render correctly passes options to template engine."""
-
         options = dict(strict=True, cache=True)
         view.engines['testplugin'] = TestTemplateEngine(options=options)
         data = dict(fruit='apple', beverage='beer')
@@ -78,9 +78,8 @@ class TestView(unittest.TestCase):
 
     def test_content_types(self):
         """Test that setting view format sets correct content-type header."""
-
         view.engines['testplugin'] = TestTemplateEngine()
-        template = "testplugin:foo"
+        template = 'testplugin:foo'
         info = dict()
         headers = {}
         view.render(info, template, headers=headers)
@@ -116,57 +115,48 @@ class TestView(unittest.TestCase):
 
 
 class TestKidView(unittest.TestCase):
-    """Test suite for turbogears.view using kid template engine."""
+    """Test suite for turbogears.view using the Kid template engine."""
 
     def setUp(self):
         """Reload the template engines before each run."""
-
         print "Loading all template engines..."
         view.load_engines()
 
     def tearDown(self):
         """Destroy the template engines after each test."""
-
         print "Destroying all template engines..."
         # We need to empty the module global but leave it in place
         view.engines.clear()
 
     def test_kid_plugins_loaded(self):
-        """Test that the kid template engine plugin is loaded."""
-
+        """Test that the Kid template engine plugin is loaded."""
         assert 'kid' in view.engines
 
     def test_UnicodeValueAppearingInATemplateIsFine(self):
         """Test that unicode values passed to template are UTF-8 encoded in output."""
-
         ustr = u"micro-eXtreme Programming ( µ XP): Embedding XP Within Standard Projects"
         info = dict(someval=ustr)
-        val = view.render(info, template="kid:turbogears.tests.simple")
-        self.assertTrue(u"Paging all " + ustr in val.decode("utf-8"))
+        val = view.render(info, template='kid:turbogears.tests.simple')
+        self.assertTrue(u"Paging all " + ustr in val.decode('utf-8'))
 
     def test_templateRetrievalByPath(self):
-        """Test that kid can find, load and compile templates."""
-
-        server_env = config.get('server.environment')
-        config.update({'server.environment' : 'development'})
+        """Test that Kid can find, load and compile templates."""
         from turbokid import kidsupport
         ks = kidsupport.KidSupport()
-        cls = ks.load_template("turbogears.tests.simple")
+        cls = ks.load_template('turbogears.tests.simple')
         assert cls is not None
         t = cls()
         t.someval = "hello"
         filled = str(t)
-        config.update({'server.environment': server_env})
         assert "groovy" in filled
         assert "html" in filled
         # the following import should not fail, if everything is working correctly:
         import turbogears.tests.simple
 
     def test_default_output_encoding(self):
-        """Test we can set kid encoding and matching content-type header is set."""
-
+        """Test we can set Kid encoding and matching content-type header is set."""
         info = dict(someval=u"Español")
-        template = "kid:turbogears.tests.simple"
+        template = 'kid:turbogears.tests.simple'
         headers = {}
         # default encoding is utf-8
         val = view.render(info, template, headers=headers)
@@ -186,10 +176,9 @@ class TestKidView(unittest.TestCase):
             config.update({'kid.encoding': kid_encoding})
 
     def test_plain_format(self):
-        """Test that kid correctly renders templates with format 'plain' or 'test'."""
-
+        """Test that Kid correctly renders templates with format 'plain' or 'test'."""
         info = dict(someval="dumbos")
-        template = "kid:turbogears.tests.simple"
+        template = 'kid:turbogears.tests.simple'
         headers = {}
         plain = view.render(info, template, headers=headers, format='plain')
         assert headers.get('Content-Type') == 'text/plain; charset=utf-8'
@@ -210,21 +199,18 @@ class TestGenshiView(unittest.TestCase):
     """Test suite for turbogears.view using Genshi template engine."""
 
     def setUp(self):
-        """Reload the template engines before each run.
-        """
+        """Reload the template engines before each run."""
         print "Loading all template engines..."
         view.load_engines()
 
     def tearDown(self):
         """Destroy the template engines after each test."""
-
         print "Destroying all template engines..."
         # We need to empty the module global but leave it in place
         view.engines.clear()
 
     def test_genshi_plugins_loaded(self):
         """Test that the Genshi template engine plugin are loaded."""
-
         assert 'genshi' in view.engines
         assert 'genshi-markup' in view.engines
         assert 'genshi-text' in view.engines
@@ -232,7 +218,6 @@ class TestGenshiView(unittest.TestCase):
     if TGGenshiTemplatePlugin:
         def test_custom_genshi_engine_loaded(self):
             """Test that view.load_engines replaced Genshi engine with our own implementation."""
-
             assert isinstance(view.engines['genshi'], TGGenshiTemplatePlugin), \
                 "Engine plugin 'genshi' was not replaced by our own implementation."
             assert isinstance(view.engines['genshi-markup'], TGGenshiTemplatePlugin), \
@@ -242,24 +227,22 @@ class TestGenshiView(unittest.TestCase):
 
     def test_UnicodeValueAppearingInATemplateIsFine(self):
         """Test that unicode values passed to template are UTF-8 encoded in output."""
-
         ustr = u"micro-eXtreme Programming ( µ XP): Embedding XP Within Standard Projects"
         info = dict(someval=ustr)
-        val = view.render(info, template="genshi:turbogears.tests.simple")
-        assert u"Paging all " + ustr in val.decode("utf-8")
+        val = view.render(info, template='genshi:turbogears.tests.simple')
+        assert u"Paging all " + ustr in val.decode('utf-8')
 
     def test_default_output_encoding(self):
         """Test we can set Genshi default encoding and headers are set correctly."""
-
         info = dict(someval=u"Español")
-        template = "genshi:turbogears.tests.simple"
+        template = 'genshi:turbogears.tests.simple'
         headers = {}
         # default encoding is utf-8
         val = view.render(info, template, headers=headers)
         assert headers.get('Content-Type') == 'text/html; charset=utf-8'
         # encoding can be changed and will be added to existing content type
         try:
-            genshi_encoding = config.get('genshi.default_encoding', "utf-8")
+            genshi_encoding = config.get('genshi.default_encoding', 'utf-8')
             config.update({'genshi.default_encoding': 'iso-8859-1'})
             # we need to reload the engines for the config change to take effect
             view.load_engines()
@@ -273,12 +256,12 @@ class TestGenshiView(unittest.TestCase):
 
     def test_default_set_encoding(self):
         """Test we can set Genshi encoding per rendering and headers are set correctly."""
-
         info = dict(someval=u"Español")
-        template = "genshi:turbogears.tests.simple"
+        template = 'genshi:turbogears.tests.simple'
         headers = {}
         # default encoding is utf-8
         val = view.render(info, template, headers=headers)
+        assert 'Paging all Español' in val
         assert headers.get('Content-Type') == 'text/html; charset=utf-8'
         # encoding can be changed and will be added to existing content type
         headers['Content-Type'] = 'text/html'
@@ -289,10 +272,9 @@ class TestGenshiView(unittest.TestCase):
 
     def test_old_text_format(self):
         """Verify that our view is able to send old text format to Genshi."""
-
         # make sure we have asked for old format in the config
-        new_syntax = config.get("genshi.new_text_syntax", False)
-        config.update({"genshi.new_text_syntax": False})
+        new_syntax = config.get('genshi.new_text_syntax', False)
+        config.update({'genshi.new_text_syntax': False})
         # we need to reload the engines for the config changes to take effect
         view.load_engines()
 
@@ -300,10 +282,10 @@ class TestGenshiView(unittest.TestCase):
         expected_result = 'Dear Florent Aide\nYour items:\n  * Beer\n  * Whisky\n'
 
         info = dict(itemlist=[u'Beer', u'Whisky'], name=u"Florent Aide")
-        template = "genshi-text:turbogears.tests.genshi_old_text_format"
+        template = 'genshi-text:turbogears.tests.genshi_old_text_format'
         headers = {}
-        val = view.render(info, template, headers=headers, format="text")
-        config.update({"genshi.new_text_syntax": new_syntax})
+        val = view.render(info, template, headers=headers, format='text')
+        config.update({'genshi.new_text_syntax': new_syntax})
 
         print "Got this:"
         print "*" * 35
@@ -317,10 +299,9 @@ class TestGenshiView(unittest.TestCase):
 
     def test_new_text_format(self):
         """Verify that our view is able to send new text format to Genshi."""
-
         # make sure we have asked for new format in the config
-        new_syntax = config.get("genshi.new_text_syntax", False)
-        config.update({"genshi.new_text_syntax": True})
+        new_syntax = config.get('genshi.new_text_syntax', False)
+        config.update({'genshi.new_text_syntax': True})
         # we need to reload the engines for the config changes to take effect
         view.load_engines()
 
@@ -331,10 +312,10 @@ class TestGenshiView(unittest.TestCase):
             name=u"Florent Aide",
             itemlist=[u'Apples', u'Bananas', u'Cherries']
         )
-        template = "genshi-text:turbogears.tests.genshi_new_text_format"
+        template = 'genshi-text:turbogears.tests.genshi_new_text_format'
         headers = {}
-        val = view.render(info, template, headers=headers, format="text")
-        config.update({"genshi.new_text_syntax": new_syntax})
+        val = view.render(info, template, headers=headers, format='text')
+        config.update({'genshi.new_text_syntax': new_syntax})
 
         print "Got this:"
         print "*" * 35
@@ -348,9 +329,8 @@ class TestGenshiView(unittest.TestCase):
 
     if TGGenshiTemplatePlugin:
         def test_genshi_default_doctypes(self):
-            """Test that Genshi uses the correct doctype with default settings.
-            """
-            template = "genshi:turbogears.tests.simple"
+            """Test that Genshi uses the correct doctype with default settings."""
+            template = 'genshi:turbogears.tests.simple'
             info = dict(someval="dumbo")
             headers = {}
             val = view.render(info, template, headers=headers)
@@ -366,9 +346,8 @@ class TestGenshiView(unittest.TestCase):
             assert not val.lstrip().startswith('<!DOCTYPE')
 
         def test_genshi_set_doctypes(self):
-            """Test that we can set the doctpye Genshi uses for each rendering.
-            """
-            template = "genshi:turbogears.tests.simple"
+            """Test that we can set the doctpye Genshi uses for each rendering."""
+            template = 'genshi:turbogears.tests.simple'
             info = dict(someval="dumbo")
             headers = {}
             val = view.render(info, template, headers=headers, doctype='html')
@@ -399,7 +378,6 @@ class TestGenshiView(unittest.TestCase):
 
 def test_template_plugin_output():
     """Test that minimal template engine test plugin returns correct output."""
-
     def extra_vars():
         return dict(tree='Rowan')
     engine = TestTemplateEngine(extra_vars_func=extra_vars)
@@ -415,11 +393,10 @@ def test_template_plugin_output():
 
 def test_template_plugin_pass_options():
     """Test that minimal template engine test plugin uses correct options."""
-
     engine = TestTemplateEngine(options=dict(cache=True, strict=False))
     output = engine.render(dict(), strict=False, renderer='fast')
     info, options = loads(output.decode('utf-8'))
-    print options
+    assert isinstance(info, dict)
     assert isinstance(options, dict)
     assert options['cache'] is True, \
         "Options passed to __init__() not used."
@@ -430,7 +407,6 @@ def test_template_plugin_pass_options():
 
 def test_get_plugin_options():
     """Test that template engine options and defaults are loaded correctly."""
-
     config.update({
         'genshi.expect': 'Spanish Inquisition',
         'genshi.spamm': 'egg',
@@ -453,7 +429,6 @@ def test_get_plugin_options():
 
 def test_cycle():
     """Test that view.base.cyle() returns correct values."""
-
     oe = view.base.cycle(('odd','even'))
     assert str(oe) == str(None)
     assert oe.next() == 'odd'
@@ -465,19 +440,16 @@ def test_cycle():
 
 def test_selector():
     """Test that view.base.selector() returns correct values."""
-
     assert view.base.selector(False) is None
     assert view.base.selector(True) == 'selected'
 
 def test_checker():
     """Test that view.base.checker() returns correct values."""
-
     assert view.base.checker(False) is None
     assert view.base.checker(True) == 'checked'
 
 def test_ipeek():
     """Test that view.base.ipeek() returns correct values."""
-
     assert view.base.checker(False) is None
     assert view.base.checker(True) == 'checked'
     seq = xrange(3, 6)
@@ -486,3 +458,4 @@ def test_ipeek():
     seq = xrange(3, 3)
     assert not view.base.ipeek(seq)
     assert list(seq) == []
+
