@@ -1,5 +1,11 @@
-import Cookie
-import cStringIO as StringIO
+"""TurboGears Test Utilities."""
+
+__all__ = ['BrowsingSession', 'DBTest', 'DBTestSA', 'DBTestSO',
+    'DummySession', 'TGTest',
+    'capture_log', 'make_app', 'make_wsgiapp', 'mount',
+    'print_log', 'get_log', 'sqlalchemy_cleanup',
+    'start_server', 'stop_server', 'unmount']
+
 import os
 import types
 import logging
@@ -7,7 +13,7 @@ import string
 import unittest
 
 import cherrypy
-from cherrypy._cphttptools import Request, Response
+from cherrypy.process.wspbus import states
 
 from webtest import TestApp
 
@@ -21,78 +27,36 @@ try:
 except ImportError:
     sqlalchemy = None
 
-from turbogears import (config, controllers, database, startup, update_config,
-    validators)
-from turbogears.identity import current_provider
-from turbogears.util import get_model, deprecated
+from turbogears import config, database, startup, update_config, validators
+from turbogears.util import get_model
 
 
-# For clean tests, remove all compiled Kid templates
-for w in os.walk('.'):
-    if not os.sep + '.' in w[0]:
-        for f in w[2]:
-            if f.endswith('.kid'):
-                f = os.path.join(w[0], f[:-3] + 'pyc')
-                if os.path.exists(f):
-                    os.remove(f)
-
-# Load test configuration
-if os.path.exists('test.cfg'):
-    # Look for a 'config' package
-    for dirpath, dirs, dummy2 in os.walk('.'):
-        basename = os.path.basename(dirpath)
-        dirname = os.path.basename(os.path.dirname(dirpath))
-        init_py = os.path.join(dirpath, '__init__.py')
-        if basename == 'config' and dirname[0] in string.ascii_letters + '_' \
-                and os.path.exists(init_py):
-            modulename = "%s.app" % dirpath[2:].replace(os.sep, ".")
-            break
-    else:
-        modulename = None
-    # XXX This is a temporary workaround, the code above to find the config
-    # package should really be improved and moved elsewhere.
-    # See http://trac.turbogears.org/ticket/2043
-    try:
-        update_config(configfile="test.cfg", modulename=modulename)
-    except ImportError, exc:
-        import warnings
-        warnings.warn("Could not import configuration from module: %s" % exc,
-            RuntimeWarning)
-        update_config(configfile="test.cfg", modulename=None)
-else:
-    database.set_db_uri("sqlite:///:memory:")
-
-config.update({'global':
-        {'autoreload.on': False, 'tg.new_style_logging': True}})
-
-
-# main testutil inferface functions for setting up & mounting a test app
-# and starting/stopping it.
-
-def mount(controller, path="/"):
-    """Mount a controller at a path.  Returns a wsgi application."""
-    if path == '/':
-        cherrypy.root = controller
-    else:
-        cherrypy.tree.mount(controller, path)
-    return make_wsgiapp()
+# Main testutil interface functions for setting up & mounting a test app
+# and starting/stopping it:
 
 def unmount():
     """Remove an application from the object traversal tree."""
-    # There's no clean way to remove a subtree under CP2, so the only use case
-    #  handled here is to remove the entire application.
-    # Supposedly, you can do a partial unmount with CP3 using:
-    #  del cherrypy.tree.apps[path]
-    cherrypy.root = None
-    cherrypy.tree.mount_points = {}
+    for app in cherrypy.tree.apps.keys():
+        del cherrypy.tree.apps[app]
+
+
+def mount(controller, path='/'):
+    """Mount a controller at a path.  Returns a WSGI application."""
+    startup.config_static()
+    if path == '/':
+        startup.config_root()
+    config.update({'environment': 'test_suite', 'log.screen': False})
+    cherrypy.tree.mount(controller, path, config=config.app)
+    return make_wsgiapp()
+
 
 def make_wsgiapp():
-    """Return a WSGI application from cherrypy's root object."""
-    wsgiapp = cherrypy._cpwsgi.wsgiApp
-    return wsgiapp
+    """Return a WSGI application from CherryPy."""
+    return cherrypy.tree
+
 
 def make_app(controller=None):
-    """Return a WebTest.TestApp instance from Cherrypy.
+    """Return a WebTest.TestApp instance from CherryPy.
 
     If a Controller object is provided, it will be mounted at the root level.
     If not, it'll look for an already mounted root.
@@ -104,32 +68,41 @@ def make_app(controller=None):
         wsgiapp = make_wsgiapp()
     return TestApp(wsgiapp)
 
-def start_server():
-    """Start the server if it's not already started."""
-    if not config.get("cp_started"):
-        cherrypy.server.start(serverClass=None, initOnly=True)
-        config.update({"cp_started" : True})
 
-    if not config.get("server_started"):
-        startup.startTurboGears()
-        config.update({"server_started" : True})
+def start_server(tg_only=True):
+    """Start the server if it's not already started.
 
-def stop_server(tg_only = False):
-    """Stop the server and unmount the application.
-
-    Use tg_only = True to leave CherryPy running (for faster tests).
+    Use tg_only=False to run the CherryPy engine as well.
 
     """
+    if not tg_only and not config.get('cp_started'):
+        cherrypy.engine.start()
+        config.update({'cp_started': True})
+
+    if not config.get('server_started'):
+        startup.start_turbogears()
+        config.update({'server_started': True})
+
+
+def stop_server(tg_only=False):
+    """Stop the server and unmount the application.
+
+    Use tg_only=True to leave CherryPy running (for faster tests).
+
+    """
+    if config.get('server_started'):
+        startup.stop_turbogears()
+        config.update({'server_started': False})
+
     unmount()
-    if config.get("cp_started") and not tg_only:
-        cherrypy.server.stop()
-        config.update({"cp_started" : False})
 
-    if config.get("server_started"):
-        startup.stopTurboGears()
-        config.update({"server_started" : False})
+    if not tg_only and config.get('cp_started'):
+        if cherrypy.engine.state != states.STOPPED:
+            cherrypy.engine.exit()
+        config.update({'cp_started': False})
 
-# misc test utility classes & functions
+
+# Miscellaneous test utility classes & functions:
 
 _currentcat = None
 
@@ -145,7 +118,7 @@ class MemoryListHandler(logging.Handler):
         self.log.append(self.format(record))
 
     def print_log(self):
-        print "\n".join(self.log)
+        print '\n'.join(self.log)
         self.log = []
 
     def get_log(self):
@@ -153,6 +126,7 @@ class MemoryListHandler(logging.Handler):
         self.log = []
         return log
 
+
 _memhandler = MemoryListHandler()
 
 def catch_validation_errors(widget, value):
@@ -168,6 +142,7 @@ def catch_validation_errors(widget, value):
         errors = {}
     return value, errors
 
+
 def capture_log(category):
     """Capture log for one category.
 
@@ -185,6 +160,7 @@ def capture_log(category):
         log.setLevel(logging.DEBUG)
         log.addHandler(_memhandler)
 
+
 def _reset_logging():
     """Manage the resetting of the loggers."""
     global _currentcat
@@ -195,6 +171,7 @@ def _reset_logging():
         log.removeHandler(_memhandler)
     _currentcat = None
 
+
 def print_log():
     """Print the log captured by capture_log to stdout.
 
@@ -204,6 +181,7 @@ def print_log():
     _reset_logging()
     _memhandler.print_log()
 
+
 def get_log():
     """Return the list of log messages captured by capture_log.
 
@@ -213,6 +191,7 @@ def get_log():
     _reset_logging()
     return _memhandler.get_log()
 
+
 def sqlalchemy_cleanup():
     database.metadata.clear()
     try:
@@ -229,12 +208,13 @@ def sqlalchemy_cleanup():
 class TGTest(unittest.TestCase):
     """A WebTest enabled unit testing class.
 
-    To use, subclass & set root to your controller object, or set app to a
+    To use, subclass and set root to your controller object, or set app to a
     webtest.TestApp instance.
 
     In your tests, use self.app to make WebTest calls.
 
     """
+
     root = None
     app = None
     stop_tg_only = False
@@ -246,7 +226,7 @@ class TGTest(unittest.TestCase):
         You should override this and make sure you have properly
         mounted a root for your server before calling super,
         or simply pass a root controller to super.
-        Otherwise the Cherrypy filters for TurboGears will not be used.
+        Otherwise the CherryPy hooks for TurboGears will not be used.
 
         """
         assert self.root or self.app, "Either self.root or self.app must be set"
@@ -258,9 +238,9 @@ class TGTest(unittest.TestCase):
 
     def tearDown(self):
         """Tear down the WebTest by stopping the server."""
-        stop_server(tg_only = self.stop_tg_only)
+        stop_server(tg_only=self.stop_tg_only)
         config.update(self.config)
-        
+
 
     def login_user(self, user):
         """Log a specified user object into the system."""
@@ -273,20 +253,21 @@ class BrowsingSession(object):
     def __init__(self):
         self.visit = None
         self.response, self.status = None, None
-        self.cookie = Cookie.SimpleCookie()
+        self.cookie = {}
         self.app = make_app()
 
-    def goto(self, *args, **kwargs):
+    def goto(self, path, headers=None, **kwargs):
+        if headers is None:
+            headers = {}
         if self.cookie:
-            headers = kwargs.setdefault('headers', {})
             headers['Cookie'] = self.cookie_encoded
-        response = self.app.get(*args, **kwargs)
+        response = self.app.get(path, headers=headers, **kwargs)
 
         # If we were given an encoding in the content type we should use it to
         # decode the response:
         ctype_parts = response.headers['Content-Type'].split(';')
         for parameter in ctype_parts[1:]:
-            attribute, value = parameter.strip().split('=')
+            value = parameter.strip().split('=', 1)[-1]
             try:
                 self.unicode_response = response.body.decode(value)
                 break
@@ -309,29 +290,6 @@ class DummySession:
     to_be_loaded = None
 
 
-class DummyRequest:
-    """A very simple dummy request."""
-
-    remote_host = "127.0.0.1"
-
-    def __init__(self, method='GET', path='/', headers=None):
-        self.headers = headers or {}
-        self.method = method
-        self.path = path
-        self.base = ''
-        self._session = DummySession()
-        self.tg_template_enginename = config.get('tg.defaultview', 'kid')
-
-    def purge__(self):
-        pass
-
-
-class DummyResponse:
-    """A very simple dummy response."""
-
-    headers = {}
-
-
 class AbstractDBTest(unittest.TestCase):
     """A database enabled unit testing class.
 
@@ -367,11 +325,12 @@ class DBTestSO(AbstractDBTest):
         constraints = list()
 
         for item in self._get_soClasses():
-            if isinstance(item, types.TypeType) and issubclass(item,
-                sqlobject.SQLObject) and item != sqlobject.SQLObject \
-                and item != InheritableSQLObject:
+            if (isinstance(item, types.TypeType)
+                    and issubclass(item, sqlobject.SQLObject)
+                    and item is not sqlobject.SQLObject
+                    and item is not InheritableSQLObject):
                 # create table without applying constraints, collect
-                # all the constaints for later creation.
+                # all the constraints for later creation.
                 # see http://sqlobject.org/FAQ.html#mutually-referencing-tables
                 # for more info
                 collected_constraints = item.createTable(ifNotExists=True,
@@ -380,7 +339,7 @@ class DBTestSO(AbstractDBTest):
                 if collected_constraints:
                     constraints.extend(collected_constraints)
 
-        # now that all tables are created, add the constaints we collected
+        # now that all tables are created, add the constraints we collected
         for postponed_constraint in constraints:
             # item is the last processed item and we borrow its connection
             item._connection.query(postponed_constraint)
@@ -388,9 +347,10 @@ class DBTestSO(AbstractDBTest):
     def tearDown(self):
         database.rollback_all()
         for item in reversed(self._get_soClasses()):
-            if isinstance(item, types.TypeType) and issubclass(item,
-                sqlobject.SQLObject) and item != sqlobject.SQLObject \
-                and item != InheritableSQLObject:
+            if (isinstance(item, types.TypeType)
+                    and issubclass(item, sqlobject.SQLObject)
+                    and item is not sqlobject.SQLObject
+                    and item is not InheritableSQLObject):
                 item.dropTable(ifExists=True, cascade=True)
 
 
@@ -404,111 +364,51 @@ class DBTestSA(AbstractDBTest):
         database.metadata.drop_all()
 
 
-# Determine which class to use for "DBTest".  Setup & teardown should behave
-# simularly regardless of which ORM you choose.
-if config.get("sqlobject.dburi"):
+# Set up the test environment (will be run when the module is imported):
+
+# For clean tests, remove all compiled Kid templates
+for w in os.walk('.'):
+    if not os.sep + '.' in w[0]:
+        for f in w[2]:
+            if f.endswith('.kid'):
+                f = os.path.join(w[0], f[:-3] + 'pyc')
+                if os.path.exists(f):
+                    os.remove(f)
+
+# Load test configuration
+if os.path.exists('test.cfg'):
+    # Look for a 'config' package
+    for dirpath, dirs, dummy2 in os.walk('.'):
+        basename = os.path.basename(dirpath)
+        dirname = os.path.basename(os.path.dirname(dirpath))
+        init_py = os.path.join(dirpath, '__init__.py')
+        if (basename == 'config' and dirname[0] in string.ascii_letters + '_'
+                and os.path.exists(init_py)):
+            modulename = '%s.app' % dirpath[2:].replace(os.sep, '.')
+            break
+    else:
+        modulename = None
+    # XXX This is a temporary workaround, the code above to find the config
+    # package should really be improved and moved elsewhere.
+    # See http://trac.turbogears.org/ticket/2043
+    try:
+        update_config(configfile='test.cfg', modulename=modulename)
+    except ImportError, exc:
+        import warnings
+        warnings.warn("Could not import configuration from module: %s" % exc,
+            RuntimeWarning)
+        update_config(configfile='test.cfg', modulename=None)
+else:
+    database.set_db_uri('sqlite:///:memory:')
+
+config.update({'global': {'engine.autoreload.on': False}})
+
+# Determine which class to use for "DBTest".
+# SetUp & TearDown should behave similarly regardless of which ORM you choose.
+
+if config.get('sqlobject.dburi'):
     DBTest = DBTestSO
-elif config.get("sqlalchemy.dburi"):
+elif config.get('sqlalchemy.dburi'):
     DBTest = DBTestSA
 else:
-    raise Exception("Unable to find sqlalchemy or sqlobject dburi")
-
-
-# deprecated functions kept for backward compability
-
-start_cp = deprecated('start_cp is superceded by start_server')(start_server)
-
-reset_cp = deprecated('reset_cp has been superceded by unmount.')(unmount)
-
-test_user = None
-
-@deprecated()
-def set_identity_user(user):
-    """Setup a user for configuring request's identity."""
-    global test_user
-    test_user = user
-
-@deprecated()
-def attach_identity(req):
-    if config.get("identity.on", False):
-        req.identity = (test_user
-            and current_provider.authenticated_identity(test_user)
-            or current_provider.anonymous_identity())
-
-@deprecated("create_request is deprecated.  See TestMigration on the TG Wiki")
-def create_request(request, method="GET", protocol="HTTP/1.1",
-        headers={}, rfile=None, clientAddress="127.0.0.1",
-        remoteHost="localhost", scheme="http"):
-    start_server()
-    if not rfile:
-        rfile = StringIO.StringIO("")
-    if type(headers) != dict:
-        headerList = headers
-    else:
-        headerList = [(key, value) for key, value in headers.items()]
-    headerList.append(("Host", "localhost"))
-    if not hasattr(cherrypy.root, "started"):
-        startup.startTurboGears()
-        cherrypy.root.started = True
-    req = Request(clientAddress, 80, remoteHost, scheme)
-    cherrypy.serving.request = req
-    attach_identity(req)
-    cherrypy.serving.response = Response()
-    req.run(" ".join((method, request, protocol)), headerList, rfile)
-    return cherrypy.serving.response
-
-# Use of createRequest will also emit deprecation warnings.
-createRequest = create_request
-
-def _return_directly(output, *args):
-    return output
-
-@deprecated("Please see the TestMigration page in the TG wiki.")
-def call(method, *args, **kw):
-    start_server()
-    output, response = call_with_request(method, DummyRequest(), *args, **kw)
-    return output
-
-@deprecated("Please see the TestMigration page in the TG wiki.")
-def call_with_request(method, request, *args, **kw):
-    """More fine-grained version of call method.
-
-    This allows using request/response.
-
-    """
-    orig_proc_output = controllers._process_output
-    controllers._process_output = _return_directly
-    cherrypy.serving.response = Response()
-    cherrypy.serving.request = request
-    if not hasattr(request, "identity"):
-        attach_identity(request)
-    output = None
-    try:
-        output = method(*args, **kw)
-    finally:
-        del cherrypy.serving.request
-        controllers._process_output = orig_proc_output
-    response = cherrypy.serving.response
-    return output, response
-
-# Public API. We don't expose deprecated functions
-__all__ = [
-    "BrowsingSession",
-    "DBTest",
-    "DBTestSA",
-    "DBTestSO",
-    "DummyRequest"
-    "DumyResponse",
-    "DummySession",
-    "TGTest",
-    "capture_log",
-    "make_app",
-    "make_wsgiapp",
-    "mount",
-    "print_log",
-    "get_log",
-    "sqlalchemy_cleanup",
-    "start_server",
-    "stop_server",
-    "unmount"
-]
+    raise Exception("Unable to find SQLAlchemy or SQLObject dburi")
