@@ -481,7 +481,7 @@ class CatWalk(turbogears.controllers.Controller):
             return ':'.join(d.split(':')[:-1])
         elif column_type == 'SODateCol':
             d = '%s' % default
-            return d.split()[0]
+            return d.split(None, 1)[0]
         return ''
 
     def column_not_none(self, column):
