@@ -475,7 +475,7 @@ def load_class(dottedpath):
             module = __import__(modulename, globals(), locals(), [classname])
         except ValueError: # Py < 2.5
             if not modulename:
-                module = __import__(__name__.split('.')[0],
+                module = __import__(__name__.split('.', 1)[0],
                     globals(), locals(), [classname])
     except ImportError:
         # properly log the exception information and return None
