@@ -18,10 +18,18 @@ from itertools import chain, imap
 from itertools import cycle as icycle
 from urllib import quote_plus
 
-import cherrypy
-import genshi
 import pkg_resources
 
+import cherrypy
+try:
+    import genshi
+except ImportError:
+    genshi = None
+try:
+    import kid
+except ImportError:
+    kid = None
+
 import turbogears
 from turbogears import identity, config
 
@@ -39,25 +47,18 @@ except ImportError:
     kid_i18n_filter = None
 
 try:
-    from genshi.filters import Translator
-    def _genshi_translator_cb(template):
-        """Callback to add a gettext Translator filter to Genshi templates.
-
-        Private module function not intended to for public use.
-
-        This is called by the Genshi template loader and adds Translator filter
-        as the first in the list of filters as is required for translation to
-        work properly.
-
-        """
-        template.filters.insert(0, Translator(gettext))
     from genshisupport import TGGenshiTemplatePlugin
 except ImportError:
-    _genshi_translator_cb = None
-    TGGenshiTemplatePlugin = None
+    TGGenshiTemplatePlugin = genshi_i18n_filter = None
+else:
+    try:
+        from genshi.filters import Translator
+    except ImportError:
+        genshi_i18n_filter = None
+    else:
+        genshi_i18n_filter = Translator(gettext)
 
-
-log = logging.getLogger("turbogears.view")
+log = logging.getLogger('turbogears.view')
 
 baseTemplates = []
 variable_providers = []
@@ -73,20 +74,20 @@ def _choose_engine(template):
     info from the configuration file.
 
     @param template: a template string as seen in the expose decorator.
-        This can be something like "kid:myproj.templates.welcome" or just
-        "myproj.templates.welcome".
+        This can be something like 'kid:myproj.templates.welcome' or just
+        'myproj.templates.welcome'.
         If a colon is found, then we try to get the engine name from the
         template string. Else we try to search it from the default engine in
         the configuration file via the `tg.defaultview` setting.
         The template name may also be just the name of the template engine,
-        as in @expose("json").
+        as in @expose('json').
     @type template: basestring or None
 
     """
     if isinstance(template, basestring):
         # if a template arg was given we try to find the engine declaration
         # in it by
-        colon = template.find(":")
+        colon = template.find(':')
         if colon > -1:
             enginename = template[:colon]
             template = template[colon+1:]
@@ -95,16 +96,16 @@ def _choose_engine(template):
             engine = engines.get(template, None)
             if engine:
                 return engine, None, template
-            enginename = config.get("tg.defaultview", "genshi")
+            enginename = config.get('tg.defaultview', 'genshi')
 
     else:
-        enginename = config.get("tg.defaultview", "genshi")
+        enginename = config.get('tg.defaultview', 'genshi')
 
     engine = engines.get(enginename, None)
 
     if not engine:
-        raise KeyError, \
-            "Template engine %s is not installed" % enginename
+        raise KeyError(
+            "Template engine %s is not installed" % enginename)
 
     return engine, template, enginename
 
@@ -119,7 +120,7 @@ def render(info, template=None, format=None, headers=None, fragment=False,
     @param template: name of the template to use
     @type template: string
 
-    @param format: "html", "xml", "text" or "json"
+    @param format: 'html', 'xml', 'text' or 'json'
     @type format: string
 
     @param headers: for response headers, primarily the content type
@@ -140,27 +141,27 @@ def render(info, template=None, format=None, headers=None, fragment=False,
         cherrypy.request.wsgi_environ['paste.testing_variables']['raw'] = info
 
     template = format == 'json' and 'json' or info.pop(
-        "tg_template", template)
+        'tg_template', template)
 
-    if not info.has_key("tg_flash"):
-        if config.get("tg.empty_flash", True):
-            info["tg_flash"] = None
+    if 'tg_flash' not in info:
+        if config.get('tg.empty_flash', True):
+            info['tg_flash'] = None
 
-    engine, template, enginename = _choose_engine(template)
+    engine, template, engine_name = _choose_engine(template)
 
     if format:
         if format == 'plain':
-            if enginename == 'genshi':
+            if engine_name == 'genshi':
                 format = 'text'
 
         elif format == 'text':
-            if enginename == 'kid':
+            if engine_name == 'kid':
                 format = 'plain'
 
     else:
-        format = enginename == 'json' and 'json' or config.get(
-            "%s.outputformat" % enginename,
-            config.get("%s.default_format" % enginename, 'html'))
+        format = engine_name == 'json' and 'json' or config.get(
+            '%s.outputformat' % engine_name,
+            config.get('%s.default_format' % engine_name, 'html'))
 
     if isinstance(headers, dict):
         # Determine the proper content type and charset for the response.
@@ -189,13 +190,15 @@ def render(info, template=None, format=None, headers=None, fragment=False,
         if mime_type_has_charset(
                 content_type) and '; charset=' not in content_type:
             charset = options.get('encoding',
-                                  get_template_encoding_default(enginename))
+                get_template_encoding_default(engine_name))
 
             if charset:
                 content_type += '; charset=' + charset
 
         headers['Content-Type'] = content_type
 
+    cherrypy.request.tg_template_engine_names = [engine_name]
+
     args, kw = adapt_call(engine.render, args=[], kw=dict(
         info=info, format=format, fragment=fragment, template=template,
         **options), start=1)
@@ -220,7 +223,7 @@ def loadBaseTemplates():
     log.debug("Loading base templates")
     for template in baseTemplates:
         engine, template, enginename = _choose_engine(template)
-        if sys.modules.has_key(template):
+        if template in sys.modules:
             del sys.modules[template]
         engine.load_template(template)
 
@@ -255,7 +258,7 @@ def selector(expression):
 
     """
     if expression:
-        return "selected"
+        return 'selected'
     else:
         return None
 
@@ -267,7 +270,7 @@ def checker(expression):
 
     """
     if expression:
-        return "checked"
+        return 'checked'
     else:
         return None
 
@@ -303,52 +306,63 @@ class UserAgent:
         self.majorVersion = None
         self.minorVersion = None
         if not useragent:
-            useragent = "unknown"
-        if useragent.find("MSIE") > -1:
-            self.browser = "msie"
-        elif useragent.find("Firefox") > -1:
-            self.browser = "firefox"
+            useragent = 'unknown'
+        if useragent.find('MSIE') > -1:
+            self.browser = 'msie'
+        elif useragent.find('Firefox') > -1:
+            self.browser = 'firefox'
         else:
             isSafari = self._re_safari.search(useragent)
             if isSafari:
-                self.browser = "safari"
+                self.browser = 'safari'
                 build = int(isSafari.group(1))
                 # this comes from:
                 # http://developer.apple.com/internet/safari/uamatrix.html
                 if build >= 412:
-                    self.majorVersion = "2"
-                    self.minorVersion = "0"
+                    self.majorVersion = '2'
+                    self.minorVersion = '0'
                 elif build >= 312:
-                    self.majorVersion = "1"
-                    self.minorVersion = "3"
+                    self.majorVersion = '1'
+                    self.minorVersion = '3'
                 elif build >= 125:
-                    self.majorVersion = "1"
-                    self.minorVersion = "2"
+                    self.majorVersion = '1'
+                    self.minorVersion = '2'
                 elif build >= 85:
-                    self.majorVersion = "1"
-                    self.minorVersion = "0"
-            elif useragent == "unknown" or useragent is None:
-                self.browser = "unknown"
+                    self.majorVersion = '1'
+                    self.minorVersion = '0'
+            elif useragent == 'unknown':
+                self.browser = 'unknown'
             else:
-                self.browser = "unknown: %s" % useragent
+                self.browser = 'unknown: %s' % useragent
 
 
 def genshi_et(element):
-    """If this is an ElementTree element, convert it to a markup stream.
+    """If this is an ElementTree element, convert it to a Genshi Markup stream.
 
     If this is a list, apply this function recursively and chain everything.
 
     """
     if hasattr(element, 'tag'):
+        if not genshi:
+            raise ImportError("Must convert ElementTree element to Genshi,"
+                " but Genshi is not installed.")
         return genshi.input.ET(element)
-
     elif isinstance(element, list):
         return chain(*imap(genshi_et, element))
-
     else:
         return element
 
 
+def kid_xml(stream):
+    """If this is a Genshi Markup stream, convert it to a Kid ElementStream."""
+    if hasattr(stream, 'render'):
+        stream = stream.render('xml')
+    if not kid:
+        raise ImportError("Must convert Genshi markup stream to Kid,"
+            " but Kid is not installed.")
+    return kid.parser.XML(stream, fragment=True)
+
+
 def stdvars():
     """Create a Bunch of variables that should be available in all templates.
 
@@ -377,9 +391,8 @@ def stdvars():
     selector
         the selector function
     session
-        the current cherrypy.session if the session_filter.on it set
-        in the app.cfg configuration file. If it is not set then session
-        will be None.
+        the current cherrypy.session if tools.sessions.on is set in the
+        app.cfg configuration file, otherwise session will be None
     tg_js
         the url path to the JavaScript libraries
     tg_static
@@ -404,7 +417,7 @@ def stdvars():
     except Exception:
         useragent = UserAgent()
 
-    if config.get('session_filter.on', None):
+    if config.get('tools.sessions.on', None):
         session = cherrypy.session
     else:
         session = None
@@ -423,17 +436,18 @@ def stdvars():
         request = cherrypy.request,
         selector = selector,
         session = session,
-        tg_js = '/' + webpath + 'tg_js',
-        tg_static = '/' + webpath + 'tg_static',
-        tg_toolbox = '/' + webpath + 'tg_toolbox',
+        tg_js = webpath + '/tg_js',
+        tg_static = webpath + '/tg_static',
+        tg_toolbox = webpath + '/tg_toolbox',
         tg_version = turbogears.__version__,
         url = turbogears.url,
         useragent = useragent,
-        widgets = '/' + webpath + 'tg_widgets',
+        widgets = webpath + '/tg_widgets',
     )
     for provider in variable_providers:
         provider(tg_vars)
     root_vars = dict()
+    root_vars['_'] = gettext
     for provider in root_variable_providers:
         provider(root_vars)
     root_vars['tg'] = tg_vars
@@ -456,12 +470,19 @@ def _get_plugin_options(plugin_name, defaults=None):
             if plugin_name.startswith(k.split('.', 1)[0]))
     else:
         options = dict()
-    for k, v in config.config.configMap["global"].items():
+    for k, v in config.items():
         if plugin_name.startswith(k.split('.', 1)[0]):
             options[k] = v
     return options
 
 
+def _get_genshi_loader_callback(template_filter):
+    """Create a Genshi template loader callback for adding the given filter."""
+    def genshi_loader_callback(template):
+        template.filters.insert(0, template_filter)
+    return genshi_loader_callback
+
+
 def load_engines():
     """Load and initialize all templating engines.
 
@@ -473,59 +494,53 @@ def load_engines():
     get = config.get
 
     engine_defaults = {
-        "cheetah.importhooks": False,
-        "cheetah.precompiled": False,
-        "genshi.default_encoding": "utf-8",
-        "genshi.default_doctype":
+        'cheetah.importhooks': False,
+        'cheetah.precompiled': False,
+        'genshi.default_doctype':
             dict(html='html-strict', xhtml='xhtml-strict', xml=None),
-        "genshi.lookup_errors": "strict",
-        "genshi.new_text_syntax": False,
-        "json.skipkeys": False,
-        "json.sort_keys": False,
-        "json.check_circular": True,
-        "json.allow_nan": True,
-        "json.indent": None,
-        "json.separators": None,
-        "json.ensure_ascii": False,
-        "json.encoding": "utf-8",
-        "json.assume_encoding": "utf-8",
-        "json.descent_bases": get("json.descent_bases",
-            get("turbojson.descent_bases", True)),
-        "kid.encoding": "utf-8",
-        "kid.assume_encoding": "utf-8",
-        "kid.precompiled": False,
-        "kid.i18n.run_template_filter": False,
-        "kid.sitetemplate": get("kid.sitetemplate",
-            get("tg.sitetemplate", "turbogears.view.templates.sitetemplate")),
-        "kid.reloadbases": False,
-        "mako.directories": [''],
-        "mako.output_encoding": "utf-8"
+        'genshi.default_encoding': 'utf-8',
+        'genshi.lookup_errors': 'strict',
+        'genshi.new_text_syntax': False,
+        'json.assume_encoding': 'utf-8',
+        'json.check_circular': True,
+        'json.descent_bases': get('json.descent_bases',
+            get('turbojson.descent_bases', True)),
+        'json.encoding': 'utf-8',
+        'json.ensure_ascii': False,
+        'json.sort_keys': False,
+        'kid.assume_encoding': 'utf-8',
+        'kid.encoding': 'utf-8',
+        'kid.precompiled': False,
+        'kid.sitetemplate': get('kid.sitetemplate',
+            get('tg.sitetemplate', 'turbogears.view.templates.sitetemplate')),
+        'mako.directories': [''],
+        'mako.output_encoding': 'utf-8'
     }
 
     # Check if the i18n filter is activated in configuration.
-    # If so we'll need to apply a translator to the Genshi filter list
-    # and/or add the Kid translation function.
-    if get("i18n.run_template_filter", False):
-        if _genshi_translator_cb:
-            callback = get("genshi.loader_callback", _genshi_translator_cb)
-            engine_defaults['genshi.loader_callback'] = callback
-        if kid_i18n_filter:
-            kid_filter = get("kid.i18n_filter", kid_i18n_filter)
-            engine_defaults["kid.i18n_filter"] = kid_filter
+    # If so we'll need to add it to the Genshi or Kid template filters.
+    if get('i18n.run_template_filter', False):
+        i18n_filter = get('genshi.i18n_filter') or genshi_i18n_filter
+        if i18n_filter:
+            callback = _get_genshi_loader_callback(i18n_filter)
+        i18n_filter = get('kid.i18n_filter') or kid_i18n_filter
+        if i18n_filter:
+            engine_defaults['kid.i18n_filter'] = i18n_filter
+            engine_defaults['kid.i18n.run_template_filter'] = True
 
     for entrypoint in pkg_resources.iter_entry_points(
-            "python.templating.engines"):
+            'python.templating.engines'):
         engine = entrypoint.load()
         plugin_name = entrypoint.name
         engine_options = _get_plugin_options(plugin_name, engine_defaults)
         log.debug("Using options for template engine '%s': %r", plugin_name,
             engine_options)
-        # Replace genshi markup template engine with our own derrived class
+        # Replace Genshi markup template engine with our own derived class
         # to support our extensions to the Buffet interface.
         # This is only a temporary measure. We should try to push our extension
         # upstream and then require a new Genshi version.
-        if plugin_name in ('genshi', 'genshi-markup') \
-                and TGGenshiTemplatePlugin:
+        if TGGenshiTemplatePlugin and plugin_name in (
+                'genshi', 'genshi-markup'):
             engines[plugin_name] = TGGenshiTemplatePlugin(
                 stdvars, engine_options)
         else:
