@@ -13,7 +13,7 @@ except ImportError:
     from sha import new as sha1
 import threading
 import time
-
+from Cookie import Morsel
 from random import random
 from datetime import timedelta, datetime
 
@@ -134,32 +134,45 @@ class VisitTool(object):
     def __call__(self, **kw):
         """Check whether submitted request belongs to an existing visit."""
 
+        # Configure the visit tool
         get = kw.get
         # Where to look for the session key in the request and in which order
         source = [s.strip().lower() for s in
             kw.get('source', 'cookie').split(',')]
         if set(source).difference(('cookie', 'form')):
-            log.warning("Unsupported 'visit.source' '%s' in configuration.")
+            log.error("Unsupported visit.source in configuration.")
         # Get the name to use for the identity cookie.
         self.cookie_name = get('cookie.name', 'tg-visit')
+        if Morsel().isReservedKey(self.cookie_name):
+            log.error("Reserved name chosen as visit.cookie.name.")
         # and the name of the request param. MUST NOT contain dashes or dots,
         # otherwise the NestedVariablesFilter will choke on it.
         visit_key_param = get('form.name', 'tg_visit')
         # TODO: The path should probably default to whatever
-        # the root is masquerading as in the event of a virtual path filter.
+        # the root is masquerading as in the event of a virtual path dispatcher.
         self.cookie_path = get('cookie.path', '/')
         # The secure bit should be set for HTTPS only sites
         self.cookie_secure = get('cookie.secure', False)
         # By default, I don't specify the cookie domain.
         self.cookie_domain = get('cookie.domain', None)
         if self.cookie_domain == 'localhost':
-            raise ValueError("localhost is"
-                " not a valid value for visit.cookie.domain. Try None instead.")
+            log.error("Invalid value 'localhost' for visit.cookie.domain."
+                " Try None instead.")
         # Use max age only if the cookie shall explicitly be permanent
         self.cookie_max_age = get('cookie.permanent',
             False) and int(get('timeout', '20')) * 60 or None
-        log.info("Visit filter initialized")
+        # Use httponly to specify that the cookie shall only be transfered
+        # in HTTP requests, and shall not be accessible through JavaScript.
+        # This is intended to mitigate some forms of cross-site scripting.
+        self.cookie_httponly = get("visit.cookie.httponly", False)
+        if self.cookie_httponly and not Morsel().isReservedKey('httponly'):
+            # Python versions < 2.6 do not support the httponly key
+            log.error("The visit.cookie.httponly setting"
+                " is not supported by this Python version.")
+            self.cookie_httponly = False
+        log.debug("Visit tool configured.")
 
+        # Establish the current visit object
         visit = current()
         if not visit:
             visit_key = None
@@ -187,6 +200,7 @@ class VisitTool(object):
                 log.debug("Created new visit with key: %s", visit_key)
             self.send_cookie(visit_key)
             set_current(visit)
+
         # Inform all the plugins that a request has been made for the current
         # visit. This gives plugins the opportunity to track click-path or
         # retrieve the visitor's identity.
@@ -237,6 +251,8 @@ class VisitTool(object):
             # 'max-age' takes precedence on standard conformant browsers
             # (this is better because it has no time sync issues)
             cookies[self.cookie_name]['max-age'] = max_age
+        if self.cookie_httponly:
+            cookies[self.cookie_name]['httponly'] = True
         log.debug("Sending visit ID cookie: %s",
             cookies[self.cookie_name].output())
 
