@@ -1,5 +1,14 @@
-import itertools
+"""Base TurboGears widget implementation"""
+
+__all__ = ['load_widgets', 'all_widgets', 'Widget', 'CompoundWidget',
+    'WidgetsList', 'register_static_directory', 'static',
+    'Resource', 'Link', 'CSSLink', 'JSLink',
+    'Source', 'CSSSource', 'JSSource',
+    'js_location', 'mochikit', 'jsi18nwidget',
+    'WidgetDescription', 'set_with_self']
+
 import os
+import itertools
 import warnings
 
 import pkg_resources
@@ -11,14 +20,7 @@ from turbogears import config, startup, view
 from turbogears.util import (setlike, to_unicode, copy_if_mutable,
     get_package_name, request_available)
 from turbogears.i18n.utils import get_locale
-from turbogears.widgets.meta import MetaWidget, load_kid_template
-
-__all__ = ['load_widgets', 'all_widgets', 'Widget', 'CompoundWidget',
-    'WidgetsList', 'register_static_directory', 'static',
-    'Resource', 'Link', 'CSSLink', 'JSLink',
-    'Source', 'CSSSource', 'JSSource',
-    'js_location', 'mochikit', 'jsi18nwidget',
-    'WidgetDescription', 'set_with_self']
+from turbogears.widgets.meta import MetaWidget, load_template
 
 
 class Enum(set):
@@ -44,6 +46,7 @@ class Enum(set):
         AttributeError
 
     """
+
     def __init__(self, *args):
         set.__init__(self, args)
 
@@ -55,25 +58,23 @@ class Enum(set):
     def __repr__(self):
         return "%s(%s)" % (self.__class__.__name__, ', '.join(map(repr, self)))
 
-# Keeps a count of the each declared widget in a WidgetsList instance
+
+# Set up a counter for every declared widget in a WidgetsList instance,
 # so their order is preserved.
 counter = itertools.count()
 
-# Load all widgets provided by the widget entry point
+
 def load_widgets():
-    for widget_mod in pkg_resources.iter_entry_points("turbogears.widgets"):
+    """Load all widgets provided by the widget entry point."""
+    for widget_mod in pkg_resources.iter_entry_points('turbogears.widgets'):
         try:
             widget_mod.load()
         except Exception, e:
-            raise ImportError, 'Error loading plugin "%s"\n%s: %s' % (
-                widget_mod, e.__class__.__name__, e)
+            raise ImportError('Error loading plugin "%s"\n%s: %s'
+                % (widget_mod, e.__class__.__name__, e))
 
 all_widgets = set()
 
-PlainHTML = load_kid_template("""
-    <html xmlns:py="http://purl.org/kid/ns#" py:replace="elements"/>
-""", modname='turbogears.widgets.plainhtml')[0]
-
 
 #############################################################################
 # Widgets base classes                                                      #
@@ -87,30 +88,36 @@ class Widget(object):
 
     Attributes you should know about:
 
-    name      : The name of the widget.
-    template  : Kid template for the widget.
-    default   : Default value for the widget.
-    css       : List of CSSLinks and CSSSources for the widget. These
-                will be automatically pulled and inserted in the
-                page template that displays it.
-    javascript: List of JSLinks and JSSources for the widget. Same as css.
-    is_named  : A property returning True if the widget has overrided it's
-                default name.
-    params    : All parameter names listed here will be treated as special
-                parameters. This list is updated by the metaclass and
-                always contains *all* params from the widget itself and
-                all it's bases. Can be used as a quick reminder of all
-                the params your widget has at it's disposal. They all
-                behave the same and have the same priorities regarding their
-                overridal. Read on...
-    params_doc: A dictionary containing 'params' names as keys and their
-                docstring as value. For documentation at the widget browser.
+    name:        The name of the widget.
+    template:    The Genshi or Kid template for the widget.
+    engine_name: The name of the templating engine to be used for
+                 rendering the template. You can also specify the engine
+                 by adding a 'genshi:' or 'kid:' prefix to the template.
+                 If the engine is not specified, and cannot be derived
+                 from a xmlns:py attribute in the template source, then
+                 the global default_engine ('genshi') will be used.
+    default:     Default value for the widget.
+    css:         List of CSSLinks and CSSSources for the widget. These
+                 will be automatically pulled and inserted in the
+                 page template that displays it.
+    javascript:  List of JSLinks and JSSources for the widget. Same as css.
+    is_named:    A property returning True if the widget has overridden its
+                 default name.
+    params:      All parameter names listed here will be treated as special
+                 parameters. This list is updated by the metaclass and
+                 always contains *all* params from the widget itself and
+                 all its bases. Can be used as a quick reminder of all
+                 the params your widget has at its disposal. They all
+                 behave the same and have the same priorities regarding their
+                 overridal. Read on...
+    params_doc:  A dictionary containing 'params' names as keys and their
+                 docstring as value. For documentation at the widget browser.
 
     All initialization parameters listed at the class attribute "params" can be
-    defined as class attributes, overriden at __init__ or at display time. They
-    will be treated as special params for the widget, which means:
+    defined as class attributes, overridden at __init__ or at display time.
+    They will be treated as special params for the widget, which means:
 
-    1) You can fix default values for them when sublcassing Widget.
+    1) You can fix default values for them when subclassing Widget.
 
     2) If passed as **params to the constructor, the will be bound automatically
        to the widget instance, taking preference over any class attributes
@@ -118,26 +125,29 @@ class Widget(object):
        attributes are safe to modify as care is taken to copy them so the class
        attribute remains unchanged.
 
-    3) They can be further overrided by passing them as keyword args to
-       display(). This will only affect that display() call in a thread-safe
-       way.
+    3) They can be further overriden by passing them as keyword arguments to the
+       display() method. This will only affect that display() method call in a
+       thread-safe way.
 
     4) A callable can be passed and it will be called automatically when sending
        variables to the template. This can be handy to pick up parameters which
        change in every request or affect many widgets simultaneously.
 
     """
+
     __metaclass__ = MetaWidget
 
     name = "widget"
     template = None
+    engine_name = None
     default = None
     css = []
     javascript = []
     params = []
     params_doc = {}
 
-    def __init__(self, name=None, template=None, default=None, **params):
+    def __init__(self, name=None, template=None, engine_name=None,
+            default=None, **params):
         """Widget initialization.
 
         All initialization has to take place in this method.
@@ -145,38 +155,45 @@ class Widget(object):
         or anytime after widget's first display.
 
         *Must* call super(MyWidget, self).__init__(*args, **kw) cooperatively,
-        unless, of course, your know what you're doing. Preferably this should
+        unless, of course, you know what you're doing. Preferably this should
         be done before any actual work is done in the method.
 
         Parameters:
 
-        name     : The widget's name. In input widgets, this will also be the
-                   name of the variable that the form will send to the
-                   controller. This is the only param that is safe to pass as a
-                   positional argument to __init__.
-        template : The template that the widget should use to display itself.
-                   Currently only Kid templates are supported. You can both
-                   initialize with a template string or with the path to a
-                   file-base template: "myapp.templates.widget_tmpl"
-        default  : Default value to display when no value is passed at display
-                   time.
-        **params : Keyword arguments specific to your widget or to any of it's
-                   bases. If listed at class attribute 'params' the will be
-                   bound automatically to the widget instance.
+        name:        The widget's name. In input widgets, this will also be the
+                     name of the variable that the form will send to the
+                     controller. This is the only param that is safe to pass as a
+                     positional argument to __init__.
+        template:    The template that the widget should use to display itself.
+                     Currently only Genshi and Kid templates are supported. You
+                     can both initialize with a template string or with the path
+                     to a file-base template: 'myapp.templates.widget_tmpl'
+        engine_name: The engine to be used for rendering the template, if not
+                     specified in the template already.
+        default:     Default value to display when no value is passed at display
+                     time.
+        **params:    Keyword arguments specific to your widget or to any of its
+                     bases. If listed at class attribute 'params' the will be
+                     bound automatically to the widget instance.
 
         Note: Do not confuse these parameters with parameters listed at
         "params". Some widgets accept parameters at the constructor which are
         not listed params, these parameter won't be passed to the template, be
-        automatically called, etc..
+        automatically called, etc.
 
         """
         self._declaration_counter = counter.next()
+
         if name:
             self.name = name
-
+        if engine_name and engine_name != self.engine_name:
+            if self.template and not template:
+                # load the template again with a different engine
+                template = self.template
+            self.engine_name = engine_name
         if template:
-            self.template_c, self.template = load_kid_template(template)
-
+            self.template_c, self.template, self.engine_name = load_template(
+                template, self.engine_name)
         if default is not None:
             self.default = default
 
@@ -185,7 +202,6 @@ class Widget(object):
             if param in params:
                 # make sure we don't keep references to mutables
                 setattr(self, param, copy_if_mutable(params.pop(param)))
-
             else:
                 # make sure we don't alter mutable class attributes
                 value, mutable = copy_if_mutable(
@@ -218,18 +234,42 @@ class Widget(object):
         pass
 
     # The following methods are needed to be a well behaved widget, however,
-    # there is rarely the need to override or extend them if inheritting
+    # there is rarely the need to override or extend them if inheriting
     # directly or indirectly from Widget.
 
     def __call__(self, *args, **params):
         """Delegate to display. Used as an alias to avoid tiresome typing."""
         return self.display(*args, **params)
 
-    def display(self, value=None, **params):
-        """Display the widget in a Kid template.
+    def _prepare_params(self, value, params):
+        """Prepare the widget params for rendering the template."""
+        for param in self.__class__.params:
+            if param in params:
+                param_value = params[param]
+                if callable(param_value):
+                    param_value = param_value()
+            else:
+                # if the param hasn't been overridden (passed as a keyword
+                # argument inside **params) put the corresponding instance
+                # value inside params.
+                param_value = getattr(self, param, None)
+            # make sure we don't pass a reference to mutables
+            params[param] = copy_if_mutable(param_value)
+        if not params.get('name'):
+            params['name'] = self.name
+        if value is None:
+            value = self.default
+            if callable(value):
+                value = value()
+        params['value'] = to_unicode(self.adjust_value(value, **params))
+        self.update_params(params)
 
-        Returns an elementtree node instance. If you need serialized output
-        in a string, call 'render' instead.
+    def display(self, value=None, **params):
+        """Display the widget in a Genshi or Kid template.
+
+        Returns a Genshi Markup stream or an ElementTree node instance,
+        depending on the template in which the widget shall be displayed.
+        If you need serialized output in a string, call 'render' instead.
 
         Probably you will not need to override or extend if inheriting from
         Widget.
@@ -243,83 +283,92 @@ class Widget(object):
                   template.
 
         """
+        engine_name = self.engine_name
         if not getattr(self, 'template_c', False):
-            warnings.warn('kid' in view.engines
+            warnings.warn(engine_name in view.engines
                 and "Widget instance %r has no template defined" % self
-                or "Trying to render a widget, but the Kid"
-                    " templating engine is not installed or not yet loaded.")
+                or "Trying to render a widget, but the %s templating engine"
+                    " is not installed or not yet loaded." % engine_name)
             return None
-
-        # logic for managing the params attribute
-        for param in self.__class__.params:
-            if param in params:
-                param_value = params[param]
-                if callable(param_value):
-                    param_value = param_value()
-
-            else:
-                # if the param hasn't been overridden (passed as a keyword
-                # argument inside **params) put the corresponding instance
-                # value inside params.
-                param_value = getattr(self, param, None)
-
-            # make sure we don't pass a reference to mutables
-            params[param] = copy_if_mutable(param_value)
-
-        if not params.get('name'):
-            params['name'] = self.name
-
-        if value is None:
-            value = self.default
-            if callable(value):
-                value = value()
-        params['value'] = to_unicode(self.adjust_value(value, **params))
-
-        self.update_params(params)
-
+        self._prepare_params(value, params)
         try:
-            transform = view.engines['kid'].transform
+            transform = view.engines[engine_name].transform
         except (KeyError, AttributeError):
             # this can happen if you render a widget before application startup
             # when view.load_engines() has not yet been called
-            raise RuntimeError("Trying to render a widget, but the Kid"
-                " templating engine is not installed or not yet loaded.")
-
-        # If the page template is Genshi, we keep track of the nesting level,
-        # because Genshi cannot display Kid's ElementTree elements directly.
-        if request_available() and request.tg_template_enginename == 'genshi':
-            display_level = getattr(request, 'tg_widgets_display_level', 0)
-            request.tg_widgets_display_level = display_level + 1
-        else:
-            display_level = None
+            raise RuntimeError(
+                "Trying to render a widget, but the %s templating engine"
+                " is not installed or not yet loaded." % engine_name)
 
+        # If the widget template is Kid, but the page template is Genshi,
+        # we need to keep track of the nesting level, because Genshi cannot
+        # display Kid's ElementTree elements directly.
+        try:
+            engine_names = request.tg_template_engine_names
+        except AttributeError:
+            request.tg_template_engine_names = engine_names = []
+        engine_names.append(engine_name)
         try:
             output = transform(params, self.template_c)
-            if display_level == 0:
-                # On the topmost level, we create a Genshi markup stream
-                # from Kid's ElementTree element to make Genshi really happy.
-                # This automatism makes wrapping widget output with ET(...)
-                # calls in Genshi page templates unnecessary.
-                output = view.genshi_et(output)
+            # convert output when switching between templating engines
+            # and make sure parameters are evaluated on time in Genshi
+            # (operating with streams doesn't work well with the path
+            # acrobatics performed for properly handling nested form fields)
+            if engine_names:
+                adapter = {
+                        'kid-genshi': view.kid_xml,
+                        'genshi-kid': view.genshi_et,
+                        'genshi-genshi': list
+                    }.get('-'.join(engine_names[-2:]))
+                if adapter:
+                    output = adapter(output)
         finally:
-            if display_level is not None:
-                request.tg_widgets_display_level = display_level
+            if engine_names:
+                engine_names.pop()
 
         return output
 
-    def render(self, value=None, format="html", **params):
+    def render(self, value=None, format='html', **params):
         """Exactly the same as display() but return serialized output instead.
 
-        Useful for debugging or to display the widget in a non-Kid template
-        like Cheetah, STAN, ...
+        Useful for debugging or to display the widget in a template other
+        than Genshi or Kid, like Cheetah, Mako, Nevow Stan, ...
 
         """
-        elem = self.display(value, **params)
-        template = PlainHTML(elements=elem)
-        return template.serialize(output=format, fragment=True)
+        engine_name = self.engine_name
+        if not getattr(self, 'template_c', False):
+            warnings.warn(engine_name in view.engines
+                and "Widget instance %r has no template defined" % self
+                or "Trying to render a widget, but the %s templating engine"
+                    " is not installed or not yet loaded." % engine_name)
+            return None
+        self._prepare_params(value, params)
+        try:
+            render = view.engines[engine_name].render
+        except (KeyError, AttributeError):
+            # this can happen if you render a widget before application startup
+            # when view.load_engines() has not yet been called
+            raise RuntimeError(
+                "Trying to render a widget, but the %s templating engine"
+                " is not installed or not yet loaded." % engine_name)
+
+        try:
+            engine_names = request.tg_template_engine_names
+        except AttributeError:
+            request.tg_template_engine_names = engine_names = []
+        engine_names.append(engine_name)
+
+        try:
+            output = render(params,
+                format=format, fragment=True, template=self.template_c)
+        finally:
+            if engine_names:
+                engine_names.pop()
+
+        return output
 
     def retrieve_javascript(self):
-        """Return the needed JavaScript ressources.
+        """Return the needed JavaScript resources.
 
         Return a setlike instance containing all the JSLinks and JSSources
         the widget needs.
@@ -331,7 +380,7 @@ class Widget(object):
         return scripts
 
     def retrieve_css(self):
-        """Return the needed CSS ressources.
+        """Return the needed CSS resources.
 
         Return a setlike instance with all the CSSLinks and CSSSources
         the widget needs.
@@ -342,18 +391,15 @@ class Widget(object):
             css.add(cssitem)
         return css
 
-    def _get_is_named(self):
+    @property
+    def is_named(self):
         """Return True if the widget has overridden its default name."""
-        if self.name != "widget":
-            return True
-        else:
-            return False
-    is_named = property(_get_is_named)
+        return self.name != "widget"
 
     def __setattr__(self, key, value):
         if self._locked:
-            raise ValueError, \
-                "It is not threadsafe to modify widgets in a request"
+            raise ValueError(
+                "It is not threadsafe to modify widgets in a request")
         else:
             return super(Widget, self).__setattr__(key, value)
 
@@ -371,6 +417,7 @@ class CompoundWidget(Widget):
     ``member_widgets`` attribute.
 
     """
+
     compound = True
     member_widgets = []
 
@@ -400,6 +447,13 @@ class CompoundWidget(Widget):
             params[member] = getattr(self, member, None)
         return super(CompoundWidget, self).display(value, **params)
 
+    def render(self, value=None, format='html', **params):
+        params["member_widgets_params"] = params.copy()
+        # logic for managing the member_widgets attribute
+        for member in self.__class__.member_widgets:
+            params[member] = getattr(self, member, None)
+        return super(CompoundWidget, self).render(value, format, **params)
+
     def update_params(self, d):
         super(CompoundWidget, self).update_params(d)
         d['value_for'] = lambda f: self.value_for(f, d['value'])
@@ -476,7 +530,8 @@ class MetaWidgetsList(type):
     as they were declared.
 
     """
-    def __new__(meta, class_name, bases, class_dict):
+
+    def __new__(mcs, class_name, bases, class_dict):
         declared_widgets = []
         for base in bases:
             declared_widgets.extend(getattr(base, 'declared_widgets', []))
@@ -488,7 +543,7 @@ class MetaWidgetsList(type):
                 # we keep the widget accessible as attribute here,
                 # i.e. we don't del class_dict[name] as we did before
         declared_widgets.sort(key=lambda w: w._declaration_counter)
-        cls = type.__new__(meta, class_name, bases, class_dict)
+        cls = type.__new__(mcs, class_name, bases, class_dict)
         cls.declared_widgets = declared_widgets
         return cls
 
@@ -500,6 +555,7 @@ class WidgetsList(list):
     with widgets ordered as attributes. Syntactic sugar for an unsweet world.
 
     """
+
     __metaclass__ = MetaWidgetsList
 
     def __init__(self, *args):
@@ -539,8 +595,8 @@ def register_static_directory(modulename, directory):
     """
     directory = os.path.abspath(directory)
     config.update({'/tg_widgets/%s' % modulename: {
-        'static_filter.on' : True,
-        'static_filter.dir': directory
+        'tools.staticdir.on' : True,
+        'tools.staticdir.dir': directory
     }})
 
 static = "turbogears.widgets"
@@ -567,7 +623,9 @@ class Resource(Widget):
     """
     order = 0
     params = ['order']
-    params_doc = {'order': "JS and CSS are sorted in this 'order' at render time"}
+    params_doc = {
+        'order': "JS and CSS are sorted in this 'order' at render time"
+    }
 
 
 class Link(Resource):
@@ -579,15 +637,15 @@ class Link(Resource):
 
     def update_params(self, d):
         super(Link, self).update_params(d)
-        d["link"] = "/%stg_widgets/%s/%s" % (startup.webpath,
+        d["link"] = "%s/tg_widgets/%s/%s" % (startup.webpath,
             self.mod, self.name)
 
     def __hash__(self):
         return hash(self.mod + self.name)
 
     def __eq__(self, other):
-        return self.mod == getattr(other, "mod", None) and \
-            self.name == getattr(other, "name", None)
+        return (self.mod == getattr(other, "mod", None)
+            and self.name == getattr(other, "name", None))
 
 
 class CSSLink(Link):
@@ -601,9 +659,8 @@ class CSSLink(Link):
     />
     """
     params = ["media"]
-    params_doc = {'media': 'Specify the media attribute for the css link tag'}
+    params_doc = {'media': 'Specify the media attribute for the CSS link tag'}
     media = "all"
-
     retrieve_css = set_with_self
 
 
@@ -618,19 +675,21 @@ class JSLink(Link):
         charset="$charset" defer="${defer and 'defer' or None}"/>
     """
     params = ["charset", "defer"]
-    params_doc = {'charset': 'The character encoding of the linked script',
-        'defer': 'If true, browser may defer execution of the script'}
+    params_doc = {
+        'charset': 'The character encoding of the linked script',
+        'defer': 'If true, browser may defer execution of the script'
+    }
     charset = None
     defer = False
-
     location = js_location.head
 
     def __init__(self, *args, **kw):
         location = kw.pop('location', None)
         super(JSLink, self).__init__(*args, **kw)
         if location:
-            #if location not in js_location:
-            #    raise ValueError, "JSLink location should be in %s" % js_location
+            if location not in js_location:
+                raise ValueError(
+                    "JSLink location should be in %s" % js_location)
             self.location = location
 
     retrieve_javascript = set_with_self
@@ -657,7 +716,7 @@ class TGMochiKit(JSLink):
 
     """
 
-    template = """<script xmlns:py="http://purl.org/kid/ns#"
+    template = """<script xmlns:py="http://genshi.edgewall.org/"
     py:for="js in javascripts" py:replace="js.display()"
     />
     """
@@ -671,7 +730,7 @@ tg.mochikit_suppress == True, but 'turbogears.mochikit' is to be included via
 'tg.include_widgets'. This is a contradiction, and mochikit_suppress overrides
 the inclusion to prevent subtle errors due to version mixing of MochiKit.""")
             return []
-        javascripts = [JSLink("tgmochikit", path)
+        javascripts = [JSLink('tgmochikit', path)
             for path in tgmochikit.get_paths()]
         return javascripts
 
@@ -679,7 +738,7 @@ the inclusion to prevent subtle errors due to version mixing of MochiKit.""")
         super(TGMochiKit, self).update_params(d)
         d['javascripts'] = self.retrieve_javascript()
 
-mochikit = TGMochiKit("tgmochikit")
+mochikit = TGMochiKit('tgmochikit')
 
 
 class JSI18NWidget(Widget):
@@ -702,8 +761,8 @@ class JSI18NWidget(Widget):
     in your application configuration.
 
     """
-    params = ['locale_catalog_providers']
 
+    params = ['locale_catalog_providers']
     locale_catalog_providers = []
 
     def __init__(self, *args, **kwargs):
@@ -735,14 +794,16 @@ class JSI18NWidget(Widget):
                         _domain = config.get('i18n.domain', 'messages')
                     else:
                         _domain = domain
-                    js_dir = pkg_resources.resource_filename(_package, _directory)
+                    js_dir = pkg_resources.resource_filename(
+                        _package, _directory)
                     register_static_directory(_package, js_dir)
                     self._package_name = _package
                     self._domain = _domain
                     self._initialized = True
                 js = []
-                for l in locale, locale[:2]:
-                    link = JSLink(self._package_name, "%s-%s.js" % (self._domain, l))
+                for loc in locale, locale[:2]:
+                    link = JSLink(
+                        self._package_name, "%s-%s.js" % (self._domain, loc))
                     if parent.linked_file_exists(link):
                         js.append(link)
                         break
@@ -756,24 +817,25 @@ class JSI18NWidget(Widget):
                 self.register_package_provider()
             self._initialized = True
         js = super(JSI18NWidget, self).retrieve_javascript()
-        js.add(JSLink("turbogears", 'js/i18n_base.js'))
+        js.add(JSLink('turbogears', 'js/i18n_base.js'))
         locale = get_locale()
         for pp in self.locale_catalog_providers:
             js.extend(pp(locale))
         return js
 
     def linked_file_exists(self, widget):
-        static_dir = config.get('static_filter.dir',
-            path='/tg_widgets/%s' % widget.mod)
-        if static_dir:
-            return os.path.exists(os.path.join(static_dir, widget.name))
-        else:
-            return False
+        widget_path = config.app.get('/tg_widgets/%s' % widget.mod)
+        if widget_path:
+            static_dir = widget_path.get('tools.staticdir.dir')
+            if static_dir:
+                return os.path.exists(os.path.join(static_dir, widget.name))
+        return False
 
 jsi18nwidget = JSI18NWidget()
 
 
 class Source(Resource):
+
     params = ["src"]
 
     def __init__(self, src, *args, **kw):
@@ -797,7 +859,6 @@ class CSSSource(Source):
     params_doc = {'src': 'The CSS source for the link',
                   'media' : 'Specify the media the css source link is for'}
     media = "all"
-
     retrieve_css = set_with_self
 
 
@@ -811,13 +872,13 @@ class JSSource(Source):
     params = ["defer"]
     params_doc = {'defer': 'If true, browser may defer execution of the script'}
     defer = False
-
     location = js_location.head
 
     def __init__(self, src, location=None, **kw):
         if location:
             if location not in js_location:
-                raise ValueError, "JSSource location should be in %s" % js_location
+                raise ValueError(
+                    "JSSource location should be in %s" % js_location)
             self.location = location
         super(JSSource, self).__init__(src, **kw)
 
@@ -828,11 +889,6 @@ class JSSource(Source):
 # Classes for supporting the toolbox widget browser                         #
 #############################################################################
 
-just_the_widget = load_kid_template("""
-    <div xmlns:py="http://purl.org/kid/ns#" py:content="for_widget.display()"/>
-""")[0]
-
-
 class MetaDescription(MetaWidget):
     """Metaclass for widget descriptions.
 
@@ -840,11 +896,12 @@ class MetaDescription(MetaWidget):
     come into existence.
 
     """
-    def __init__(cls, name, bases, dct):
-        super(MetaDescription, cls).__init__(name, bases, dct)
-        register = dct.get("register", True)
-        if name != "WidgetDescription" and register:
-            all_widgets.add(cls)
+
+    def __init__(mcs, name, bases, dct):
+        super(MetaDescription, mcs).__init__(name, bases, dct)
+        register = dct.get('register', True)
+        if name != 'WidgetDescription' and register:
+            all_widgets.add(mcs)
 
 
 class WidgetDescription(CompoundWidget):
@@ -855,34 +912,38 @@ class WidgetDescription(CompoundWidget):
     platform while developing them.
 
     """
+
     __metaclass__ = MetaDescription
 
-    template = just_the_widget
+    template = """
+    <div xmlns:py="http://genshi.edgewall.org/"
+        py:content="for_widget.display()"/>
+    """
     for_widget = None
     member_widgets = ["for_widget"]
     show_separately = False
 
-    def _get_name(self):
+    @property
+    def name(self):
         return self.for_widget_class.__name__
-    name = property(_get_name)
 
-    def _get_widget_class(self):
+    @property
+    def for_widget_class(self):
         return self.for_widget.__class__
-    for_widget_class = property(_get_widget_class)
 
-    def _get_description(self):
+    @property
+    def description(self):
         return self.for_widget_class.__doc__
-    description = property(_get_description)
 
-    def _get_full_class_name(self):
+    @property
+    def full_class_name(self):
         cls = self.for_widget_class
         return "%s.%s" % (cls.__module__, cls.__name__)
-    full_class_name = property(_get_full_class_name)
 
-    def _get_source(self):
+    @property
+    def source(self):
         import inspect
         return inspect.getsource(self.__class__)
-    source = property(_get_source)
 
     def retrieve_css(self):
         return self.for_widget.retrieve_css()
@@ -892,15 +953,17 @@ class WidgetDescription(CompoundWidget):
 
 
 class CoreWD(WidgetDescription):
+
     register = False
 
-    def _get_full_class_name(self):
+    @property
+    def full_class_name(self):
         cls = self.for_widget_class
         return "turbogears.widgets.%s" % (cls.__name__)
-    full_class_name = property(_get_full_class_name)
 
 
 class RenderOnlyWD(WidgetDescription):
+
     register = False
     template = """
     <div>
@@ -921,20 +984,25 @@ class RenderOnlyWD(WidgetDescription):
 #############################################################################
 
 class CSSLinkDesc(CoreWD, RenderOnlyWD):
+
     name = "CSS Link"
-    for_widget = CSSLink("turbogears", "css/yourstyle.css")
+    for_widget = CSSLink('turbogears', 'css/yourstyle.css')
 
 
 class JSLinkDesc(CoreWD, RenderOnlyWD):
+
     name = "JS Link"
-    for_widget = JSLink("turbogears", "js/yourscript.js")
+    for_widget = JSLink('turbogears', 'js/yourscript.js')
 
 
 class CSSSourceDesc(CoreWD, RenderOnlyWD):
+
     name = "CSS Source"
-    for_widget = CSSSource("""body { font-size:12px; }""")
+    for_widget = CSSSource("body { font-size:12px; }")
 
 
 class JSSourceDesc(CoreWD, RenderOnlyWD):
+
     name = "JS Source"
     for_widget = JSSource("document.title = 'Hello World';")
+
