@@ -1,16 +1,12 @@
-"""Generic widget to present and manipulate data in a grid (tabular) form.
+"""Generic widget to present and manipulate data in a grid (tabular) form."""
 
-See also: http://trac.turbogears.org/turbogears/wiki/DataGridWidget
-
-"""
+__all__ = ['DataGrid', 'PaginateDataGrid']
 
 from turbogears.widgets import Widget, CSSLink, static
 from turbogears.widgets.base import CoreWD
 
 NoDefault = object()
 
-__all__ = ["DataGrid", "PaginateDataGrid"]
-
 
 class DataGrid(Widget):
     """Generic widget to present and manipulate data in a grid (tabular) form.
@@ -25,6 +21,7 @@ class DataGrid(Widget):
     dynamically, by via 'fields' key.
 
     """
+
     css = [CSSLink(static, "grid.css")]
     template = "turbogears.widgets.templates.datagrid"
     fields = None
@@ -40,6 +37,7 @@ class DataGrid(Widget):
           - attrwrapper, which is a function returning an object's attribute
 
         """
+
         def __init__(self, name,
                 getter=None, title=None, options=None, attrwrapper=None):
             if name is None:
@@ -56,14 +54,17 @@ class DataGrid(Widget):
             self.name = name
             self.title = title is None and name.capitalize() or title
             self.options = options or {}
+
         def get_option(self, name, default=NoDefault):
             if name in self.options:
                 return self.options[name]
             if default is NoDefault: # no such key and no default is given
                 raise KeyError(name)
             return default
+
         def get_field(self, row):
             return self.getter(row)
+
         def __str__(self):
             return "<DataGrid.Column %s>" % self.name
 
@@ -74,16 +75,20 @@ class DataGrid(Widget):
         simple static ones, and also allows nested access.
 
         """
+
         def __init__(self, name):
             if not isinstance(name, (int, str)):
                 raise ValueError('attribute name must be'
                     ' an integer index or a string attribute')
             self.name = name
+
         def __call__(self, obj):
             if isinstance(obj, (dict, list, tuple)):
                 return obj[self.name]
             for name in self.name.split('.'):
                 obj = getattr(obj, name)
+                if obj is None:
+                    break
             return obj
 
     def __init__(self, fields=None, **kw):
@@ -200,3 +205,10 @@ class PaginateDataGridDesc(CoreWD):
         from cherrypy import request
         request.paginate = self.Paginate()
         return super(PaginateDataGridDesc, self).display(*args, **kw)
+
+    def render(self, *args, **kw):
+        # activate paginate var provider
+        import turbogears.paginate
+        from cherrypy import request
+        request.paginate = self.Paginate()
+        return super(PaginateDataGridDesc, self).render(*args, **kw)
