@@ -70,7 +70,7 @@ def path_from_item(item, base_path=None):
         index = 0
         for key in path:
             if '-' in key:
-                key, num = key.split('-')
+                key, num = key.split('-', 1)
                 path[index] = key, int(num)
             else:
                 path[index] = key, None
